Package: vsn
Version: 3.81.0
Title: Variance stabilization and calibration for microarray data
Authors@R: c(
    person("Wolfgang", "Huber", email="wolfgang.huber@embl.org", role = c("aut","cre")),
    person("Anja", "von Heydebreck", role = c("aut")),
    person("Dennis", "Kostka", role = c("ctb")),
    person("David", "Kreil", role = c("ctb")),
    person("Hans-Ulrich", "Klein", role = c("ctb")),
    person("Robert", "Gentleman", role = c("ctb")),
    person("Deepayan", "Sarkar", role = c("ctb")),
    person("Gordon", "Smyth", role = c("ctb")),
    person("Federal Ministry of Research, Technology and Space of Germany, DHGP", role="fnd"))
Maintainer: Wolfgang Huber <wolfgang.huber@embl.org>
Depends: R (>= 4.0.0), methods, Biobase
Imports: affy, limma, lattice, ggplot2
Suggests: affydata, hgu95av2cdf, BiocStyle, knitr, rmarkdown, dplyr,
        testthat, hexbin
Description: The package implements a method for normalising microarray intensities from single- and 
        multiple-color arrays. It can also be used
	for data from other technologies, as long as they have similar format. The method uses a
	robust variant of the maximum-likelihood estimator for an additive-multiplicative error
	model and affine calibration. The model incorporates data calibration step (a.k.a.
	normalization), a model for the dependence of the variance on the mean intensity and a
        variance stabilizing data transformation. Differences between
        transformed intensities are analogous to "normalized
        log-ratios". However, in contrast to the latter, their
        variance is independent of the mean, and they are usually more
        sensitive and specific in detecting differential
        transcription.
Reference: [1] Variance stabilization applied to microarray data
        calibration and to the quantification of differential
        expression, Wolfgang Huber, Anja von Heydebreck, Holger
        Sueltmann, Annemarie Poustka, Martin Vingron; Bioinformatics
        (2002) 18 Suppl1 S96-S104. [2] Parameter estimation for the
        calibration and variance stabilization of microarray data,
        Wolfgang Huber, Anja von Heydebreck, Holger Sueltmann,
        Annemarie Poustka, and Martin Vingron; Statistical Applications
        in Genetics and Molecular Biology (2003) Vol. 2 No. 1, Article
        3; http://www.bepress.com/sagmb/vol2/iss1/art3.
License: Artistic-2.0
URL: https://github.com/Huber-group-EMBL/vsn
BugReports: https://github.com/Huber-group-EMBL/vsn/issues
biocViews: Microarray, OneChannel, TwoChannel, Preprocessing
VignetteBuilder: knitr
Collate: AllClasses.R AllGenerics.R vsn2.R vsnLogLik.R justvsn.R
        methods-vsnInput.R methods-vsn.R methods-vsn2.R
        methods-predict.R RGList_to_NChannelSet.R meanSdPlot-methods.R
        plotLikelihood.R normalize.AffyBatch.vsn.R sagmbSimulateData.R
        zzz.R
RoxygenNote: 7.3.3
git_url: https://git.bioconductor.org/packages/vsn
git_branch: devel
git_last_commit: 315f622
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-29 01:49:20 UTC; biocbuild
Author: Wolfgang Huber [aut, cre],
  Anja von Heydebreck [aut],
  Dennis Kostka [ctb],
  David Kreil [ctb],
  Hans-Ulrich Klein [ctb],
  Robert Gentleman [ctb],
  Deepayan Sarkar [ctb],
  Gordon Smyth [ctb],
  Federal Ministry of Research, Technology and Space of Germany, DHGP
    [fnd]

useDynLib("vsn")

import("Biobase")
import("ggplot2")

importFrom("grDevices", "colorRampPalette")
importFrom("graphics", "matplot")
importFrom("methods", ".valueClassTest", "as", "is", "new", "slot", "validObject", "show")
importFrom("stats", "df", "median", "quantile", "rgamma", "rnorm", "runif", "predict", "logLik",
           "coef", "coefficients")
importFrom("lattice", "levelplot")
importFrom("affy", "rma", "normalize.AffyBatch.methods", "upDate.normalize.AffyBatch.methods",
           "indexProbes", "intensity", "intensity<-")

importClassesFrom("limma", "RGList")
importClassesFrom("affy", "AffyBatch")

exportClasses("vsn", "vsnInput")

export("vsnMatrix", "justvsn", "vsnrma", "sagmbAssess", "sagmbSimulateData",
       "plotVsnLogLik", "scalingFactorTransformation")

exportMethods("vsn2", "meanSdPlot", "predict", "logLik", "coef", "coefficients",
              "show", "exprs", "nrow", "ncol", "dim", "coerce")

inst/vignettes/convergence2.Rnw is the most comprehensive test suite. It assesses the
accuracy of fitted transformations versus the known true ones, using
simulated data.

Then there are some more basic tests and debugging platforms for the
ML estimators:

testmlest.R is a script that checks whether the ML estimate of the
parameters a and b, if mu_k and sigma are known, is correct, based on
simulated data. It also plots the likelihood landscape around the 
true parameters.

testderiv.R tests whether the explicit gradient of the neg
loglikelihood that is used by in the vsn code accords with the
numerical differentation of the neg loglikelihood.

testprofiling.R tests whether the ML estimate of the parameters a and
b, if mu_k and sigma are known, is correct, based on a previous
profile LL fit.

#------------------------------------------------------------
# methods related to the class 'vsn'
#------------------------------------------------------------
setMethod("nrow", signature("vsn"), function(x) length(x@mu))
setMethod("ncol", signature("vsn"), function(x) switch(x@calib, affine = dim(x@coefficients)[2], none = nrow(x@hx), stop("Invalid 'calib' slot")))
setMethod("dim",  signature("vsn"), function(x) c(nrow(x), ncol(x)))

setMethod("show", signature("vsn"),
  function(object) {
    cat(class(object), sprintf("object for %d features and %d samples.\n",
      nrow(object), ncol(object)))
    if(length(object@strata)>0)
      cat(sprintf("strata: %d level%s.\n", nlevels(object@strata), c("", "s")[1+(nlevels(object@strata)>1)]))
    cat(sprintf("sigsq=%g\n", round(object@sigsq, 3)))
    if(nrow(object@hx)>0)
      cat(sprintf("hx: %d x %d matrix.\n", nrow(object@hx), ncol(object@hx)))
  })

setMethod("[", "vsn",
  function(x, i, j, ..., drop=FALSE) {
    stopifnot(missing(j), length(list(...))==0, !drop)

    x@mu = x@mu[i,drop=FALSE]
    
    if(length(x@strata)>0)
      x@strata = x@strata[i,drop=FALSE]
    if(nrow(x@hx)>0) 
      x@hx = x@hx[i,,drop=FALSE]
    
    return(x)
  })
       
setMethod("coef", signature(object="vsn"),
          function(object) object@coefficients)
setMethod("coefficients", signature(object="vsn"),
          function(object) object@coefficients)

setMethod("exprs", signature(object="vsn"),
          function(object) object@hx)

library(marrayNorm)
library(vsn)
data(swirl)

if(!exists("nyloess")) {
  ## stratified vsn:
  nystrat = vsn(swirl, strata=as.integer(maPrintTip(swirl)))

  normparams = nystrat@description@preprocessing$vsnParams
  Mvsnstrat = nystrat@exprs[,1:4] - nystrat@exprs[,5:8]

  ## unstratified vsn
  ny = vsn(dat)
  Mvsn = ny@exprs[,1:4] - ny@exprs[,5:8]

  ## print-tip loess
  nyloess = maNorm(swirl, norm="p")
}

nrpt <- max(maPrintTip(swirl))

## boxplots
x11()
par(mfrow=c(4,2))
for (j in 1:4) {
  boxplot(Mvsnstrat[,j] ~maPrintTip(swirl), col=1:nrpt, main = paste("Array", j, "strat"))
  boxplot(Mvsn[,j] ~maPrintTip(swirl), col=1:nrpt, main = paste("Array", j))
}

dev.copy(pdf, file="swirl-boxplots.pdf", width=7, height=12)
dev.off()

x11()
par(mfrow=c(4,2))
## compare stratified and unstratified vsn, and stratified vsn and print-tip loess:
for (j in 1:4) {
  plot(Mvsn[,j], Mvsnstrat[,j], xlab="vsn unstrat", ylab="vsn strat", col=maPrintTip(swirl), pch=".")
  plot(nyloess@maM[,j], Mvsnstrat[,j], xlab="loess strat", ylab="vsn strat", col=maPrintTip(swirl), pch=".")
}
dev.copy(pdf, file="swirl-scpvsn.pdf", width=7, height=12)
dev.off()

#look at the parameters for the 16 print-tips across arrays:
x11()
par(mfrow=c(1,2))
boxplot(as.data.frame(matrix(normparams[1:128], ncol=16)), main="offsets")
boxplot(as.data.frame(matrix(normparams[129:256], ncol=16)), main="factors")
dev.copy(pdf, file="swirl-params.pdf", width=7, height=4)
dev.off()

## ----setup, echo = FALSE, results = "hide"------------------------------------
options(signif = 3, digits = 3)
knitr::opts_chunk$set(tidy = FALSE, cache = TRUE, autodep = TRUE, fig.height = 5.5,
                      message = FALSE, error = FALSE, warning = TRUE)
set.seed(0xdada)

## ----overv1, results = "hide"-------------------------------------------------
library("vsn")
data("kidney")
xnorm = justvsn(kidney)

## ----overv2-------------------------------------------------------------------
M = exprs(xnorm)[,1] - exprs(xnorm)[,2]

## ----overv3, results = "hide"-------------------------------------------------
fit = vsn2(kidney)
ynorm = predict(fit, kidney)

## ----overv4, echo = FALSE, results = "hide"-----------------------------------
stopifnot(
  max(abs(exprs(xnorm) - exprs(ynorm))) < 1e-16, 
  max(abs(exprs(xnorm) - exprs(fit)))   < 1e-16,
  identical(dim(exprs(xnorm)), dim(exprs(ynorm))), 
  identical(dim(exprs(xnorm)), dim(exprs(fit))))

## ----nkid-scp, results = "hide", fig.cap = paste0("Scatterplots of the kidney example data, which were obtained from a two-color cDNA array by quantitating spots and subtracting a local background estimate. a) unnormalised and $\\log_2$-transformed. b) normalised and transformed with VSN. Panel b shows the data from the complete set of ", nrow(kidney), " spots on the array, Panel a only the ", sum(0==rowSums(exprs(kidney)<=0)), " spots for which both red and green net intensities were greater than 0. Those spots which are missing in Panel a are coloured in orange in Panel b."), fig.height = 4, fig.wide = TRUE----
library("ggplot2")
allpositive = (rowSums(exprs(kidney) <= 0) == 0)

df1 = data.frame(log2(exprs(kidney)[allpositive, ]),
                 type = "raw",
		 allpositive = TRUE)
df2 = data.frame(exprs(xnorm),
                 type = "vsn",
		 allpositive = allpositive)
df = rbind(df1, df2)
names(df)[1:2] = c("x", "y") 

ggplot(df, aes(x, y, col = allpositive)) + geom_hex(bins = 40) +
  coord_fixed() + facet_grid( ~ type)

## ----nkid-sdmean1, fig.cap = "Standard deviation versus rank of the mean", fig.height = 4, fig.small = TRUE----
meanSdPlot(xnorm, ranks = TRUE)

## ----nkid-sdmean2, fig.cap = "Standard deviation versus mean", fig.height = 4, fig.small = TRUE----
meanSdPlot(xnorm, ranks = FALSE) 

## ----nkid-histM, fig.cap = "Histogram of generalised log-ratios for the kidney example data.", fig.small = TRUE----
hist(M, breaks = 100, col = "#d95f0e")

## ----lymphoma-----------------------------------------------------------------
data("lymphoma")
dim(lymphoma)

## ----pDatalym-----------------------------------------------------------------
pData(lymphoma)

## ----lymjustvsn, results = "hide"---------------------------------------------
lym = justvsn(lymphoma)

## ----lym-sdmean, fig.cap = "Standard deviation versus rank of the mean for the lymphoma example data", fig.height = 4, fig.small = TRUE----
meanSdPlot(lym)

## ----lym-M, fig.cap = "MA-plot for slide DLCL-0032 from the lymphoma example data. A false-colour representation of the data point density is used, in addition the 100 data points in the least dense regions are plotted as dots.", fig.small = TRUE----
iref = seq(1, 15, by=2)
ismp = seq(2, 16, by=2)
M = exprs(lym)[,ismp]-exprs(lym)[,iref] 
A =(exprs(lym)[,ismp]+exprs(lym)[,iref])/2
colnames(M) = lymphoma$sample[ismp]
colnames(A) = colnames(M)

j = "DLCL-0032"
smoothScatter(A[,j], M[,j], main=j, xlab="A", ylab="M", pch=".")
abline(h=0, col="red")

## ----loadaffy1, results = "hide", warning = FALSE-----------------------------
library("affydata")
data("Dilution")
d_vsn = vsnrma(Dilution)

## ----loadaffy2, results = "hide"----------------------------------------------
d_rma = rma(Dilution)

## ----affy, fig.height=3.75, fig.wide = TRUE, fig.cap = paste0("Results of `vsnrma` and `rma` on the Dilution data. Array 1 was hybridised with ", pData(Dilution)$liver[1], "$\\mu$g RNA from liver, Array 3 with ", pData(Dilution)$liver[3], "$\\mu$g of the same RNA.")----
par(mfrow = c(1, 3))
ax = c(2, 16)

plot(exprs(d_vsn)[,c(1,3)], main = "vsn: array 1 vs 3", asp = 1, xlim = ax, ylim = ax, pch = ".")

plot(exprs(d_rma)[,c(1,3)], main = "rma: array 1 vs 3", asp = 1, xlim = ax, ylim = ax, pch = ".")

plot(exprs(d_rma)[,1], exprs(d_vsn)[,1], 
     xlab = "rma", ylab = "vsn", asp = 1, xlim = ax, ylim = ax, main = "array 1", pch = ".")
abline(a = 0,  b =1, col = "#ff0000d0")

## ----loadlimma, results = "hide"----------------------------------------------
library("limma")
wg = which(lymphoma$dye=="Cy3")
wr = which(lymphoma$dye=="Cy5")

lymRG = new("RGList", list(
  R=exprs(lymphoma)[, wr],
  G=exprs(lymphoma)[, wg]))

lymNCS = justvsn(lymRG)

## ----lymNCS-------------------------------------------------------------------
lymNCS

## ----addmeta------------------------------------------------------------------
vmd = data.frame(
  labelDescription = I(c("array ID", "sample in G", "sample in R")),
  channel = c("_ALL", "G", "R"),
  row.names = c("arrayID", "sampG", "sampR"))

arrayID = lymphoma$name[wr]
stopifnot(identical(arrayID, lymphoma$name[wg])) 

## remove sample number suffix 
sampleType = factor(sub("-.*", "", lymphoma$sample)) 

v = data.frame(
  arrayID = arrayID,
  sampG   = sampleType[wg],
  sampR   = sampleType[wr])
v
              
adf = new("AnnotatedDataFrame", 
  data = v,
  varMetadata = vmd)

phenoData(lymNCS) = adf 

## ----lymM---------------------------------------------------------------------
lymM = (assayData(lymNCS)$R - 
        assayData(lymNCS)$G)

## ----design-------------------------------------------------------------------
design = model.matrix( ~ lymNCS$sampR)
lf = lmFit(lymM, design[, 2, drop=FALSE])
lf = eBayes(lf)

## ----limma, fig.wide = TRUE, fig.cap = paste("Left: histogram of p-values from the moderated $t$-test between the",  levels(lymNCS$sampR)[1], "and",  levels(lymNCS$sampR)[2], "groups on the `lymM` values. Right: M-values for the 5 genes with the smallest p-values.")----
par(mfrow=c(1,2))
hist(lf$p.value, breaks = 100, col="orange") 
pdat = t(lymM[order(lf$p.value)[1:5],])
matplot(pdat, lty = 1, type = "b", lwd = 2, col=hsv(seq(0,1,length=5), 0.7, 0.8), 
  ylab = "M", xlab = "arrays")

## ----makebg-------------------------------------------------------------------
rndbg = function(x, off, fac)
   array(off + fac * runif(prod(dim(x))), dim = dim(x))

lymRGwbg    = lymRG
lymRGwbg$Rb = rndbg(lymRG, 100, 30)
lymRGwbg$Gb = rndbg(lymRG,  50, 20)

## ----justvsnwbg, message = FALSE----------------------------------------------
lymESwbg = justvsn(lymRGwbg[, 1:3], backgroundsubtract=TRUE)

## ----pinId--------------------------------------------------------------------
ngr = ngc = 4L
nsr = nsc = 24L
arrayGeometry = data.frame(
  spotcol = rep(1:nsc, times = nsr*ngr*ngc),
  spotrow = rep(1:nsr, each = nsc, times=ngr*ngc),
  pin = rep(1:(ngr*ngc), each = nsr*nsc))

## ----strata, results = "hide"-------------------------------------------------
EconStr = justvsn(lymRG[, 1], strata = arrayGeometry$pin)

## ----nostrata, results = "hide"-----------------------------------------------
EsenzaStr = justvsn(lymRG[, 1])

## ----figstrata2, fig.small = TRUE, fig.cap = "Scatterplot of normalised and transformed intensities for the red channel of Array 1. Values on the $x$-axis correspond to normalisation without strata (`EsenzaStr`), values on the $y$-axis to normalisation with strata (`EconStr`). The different colours correspond to the 16 different strata."----
j = 1
plot(assayData(EsenzaStr)$R[,j],
     assayData(EconStr)$R[,j],
     pch = ".", asp = 1, 
     col = hsv(seq(0, 1, length=ngr*ngc), 
       0.8, 0.6)[arrayGeometry$pin], 
     xlab = "without strata", 
     ylab = "print-tip strata",
     main = sampleNames(lymNCS)$R[j])

## ----miss1--------------------------------------------------------------------
lym2 = lymphoma
nfeat = prod(dim(lym2))
wh = sample(nfeat, nfeat/10)
exprs(lym2)[wh] = NA
table(is.na(exprs(lym2)))

## ----miss2, results = "hide"--------------------------------------------------
fit1 = vsn2(lymphoma, lts.quantile=1)
fit2 = vsn2(lym2, lts.quantile=1)

## ----miss, fig.wide = TRUE, fig.cap="Scatterplots of fitted parameters, values on the $x$-axis correspond to normalisation without missing data (`fit1`), values on the $y$-axis to normalisation with around 10% missing data (`fit2`)."----
par(mfrow=c(1,2))
for(j in 1:2){
  p1 = coef(fit1)[,,j]
  p2 = coef(fit2)[,,j]
  d  = max(abs(p1-p2))
  stopifnot(d < c(0.05, 0.03)[j])
  plot(p1, p2, pch = 16, asp = 1,
    main = paste(letters[j], 
      ": max diff=", signif(d,2), sep = ""),
    xlab = "no missing data",
    ylab = "10% of data missing")
  abline(a = 0, b = 1, col = "blue")
}

## ----spikein, warning = FALSE, message = FALSE--------------------------------
spikeins = 100:200
spfit = vsn2(kidney[spikeins,], lts.quantile=1)
nkid = predict(spfit, newdata=kidney)

## ----ref1, results = "hide"---------------------------------------------------
ref = vsn2(lymphoma[, ismp[1:7]])

## ----ref2, results = "hide"---------------------------------------------------
f8 = vsn2(lymphoma[, ismp[8]], reference = ref)

## ----ref3, results = "hide"---------------------------------------------------
fall = vsn2(lymphoma[, ismp])

## ----ref4---------------------------------------------------------------------
coefficients(f8)[,1,]
coefficients(fall)[,8,]

## ----ref, fig.small = TRUE, fig.cap = "Scatterplot of normalised intensities after normalisation by reference ($x$-axis, `f8`) and joint normalisation ($y$-axis, `fall`). There is good agreement."----
plot(exprs(f8), exprs(fall)[,8], pch = ".", asp = 1)
abline(a = 0, b = 1, col = "red")

## ----hiddenchecks, echo = FALSE-----------------------------------------------
stopifnot(length(ismp)==8L)
maxdiff = max(abs(exprs(f8) - exprs(fall)[,8])) 
if(maxdiff>0.3)
  stop(sprintf("maxdiff is %g", maxdiff))

## ----nkid-calib1--------------------------------------------------------------
coef(fit)[1,,]

## ----nkid-calib2--------------------------------------------------------------
bkid = kidney
exprs(bkid)[,1] = 0.25*(500+exprs(bkid)[,1])

## ----nkid-calib3, results = "hide"--------------------------------------------
bfit = vsn2(bkid)

## ----nkid-calib4, warning = FALSE, fig.cap = "Scatterplot for the badly biased `bkid` data: log-log scale.", fig.small = TRUE----
plot(exprs(bkid), main = "raw", pch = ".", log = "xy")

## ----nkid-calib5, warning = FALSE, fig.cap = "Scatterplot for the badly biased `bkid` data: after calibration and transformation with vsn.", fig.small = TRUE----
plot(exprs(bfit), main = "vsn", pch = ".")
coef(bfit)[1,,]

## ----vsnQ1, results = "hide", fig.cap = "Scatterplot between the red and green intensities of the array of the lymphoma dataset after quantile normalisation followed by VSN variance stabilisation without calibration. ", fig.small = TRUE----
lym_q = normalizeQuantiles(exprs(lymphoma))
lym_qvsn = vsn2(lym_q, calib="none")
plot(exprs(lym_qvsn)[, 1:2], pch=".", main="lym_qvsn")

## ----vsnQ2, results = "hide", fig.cap = "Comparison of the values calculated in this section, for CH1 of the first array, to those of VSN variance stabilisation with affine calibration (`lym` was computed in Section \\@ref(sec:lymphoma)).", fig.small = TRUE----
plot(exprs(lym)[,1], exprs(lym_qvsn)[, 1], 
  main="lym_qvsn vs lym", pch=".", 
  ylab="lym_qvsn[,1]", xlab="lym[,1]")

## ----calcshrink, echo = FALSE, results = "hide"-------------------------------
log2.na = function(x){
  w = which(x>0)
  res = rep(as.numeric(NA), length(x))
  res[w] = log2(x[w])
  return(res)
}
  
fc = 0.5                    ## true fold change
x2 = seq(0.5, 15, by=0.5)   ## 'true values' in sample 1
x1 = x2/fc                  ## 'true values' in sample 2
m = s = numeric(length(x1))
n  = 10000
sa = 1
b  = 1
sb = 0.1
sdeta = 0.1
for(i in 1:length(x1)){
  z1 = sa*rnorm(n)+x1[i]*b*exp(sb*rnorm(n))
  z2 = sa*rnorm(n)+x2[i]*b*exp(sb*rnorm(n))
  if(i%%2==1) {
    q = log2.na(z1/z2)
    m[i] = mean(q, na.rm=TRUE)
    s[i] = sd(q, na.rm=TRUE)
  } else {
    h = (asinh(z1/(sa*b/sb))-asinh(z2/(sa*b/sb)))/log(2)
    m[i] = mean(h)
    s[i] = sd(h)
  }
}

colq = c("black", "blue")
lty  = 1
pch  = c(20,18)
cex  = 1.4
lwd  = 2

## ----shrink, results = "hide", echo = FALSE, fig.cap = "The shrinkage property of the generalised log-ratio.  Blue diamonds and error bars correspond to mean and standard deviation of the generalised log-ratio $h$, as obtained from VSN, and black dots and error bars to the standard log-ratio $q$ (both base 2).", fig.small = TRUE----
mai = par("mai"); mai[3]=0; par(mai = mai)
plot(x2, m, ylim=c(-2, 3.5), type="n", xlab=expression(x[2]), ylab="")
abline(h=-log2(fc), col="red", lwd=lwd, lty=1)
abline(h=0, col="black", lwd=1, lty=2)
points(x2, m, pch=pch, col=colq, cex=cex)
segments(x2, m-2*s, x2, m+2*s, lwd=lwd, col=colq, lty=lty)
legend(8.75, -0.1, c("q","h"), col=colq, pch=pch, lty=lty, lwd=lwd)

## ----figgraph, results = "hide", echo = FALSE, fig.cap = "Graphs of the functions. $y=\\log_2(x)$, $y=$glog$_2(x,c)=\\log_2(x+\\sqrt{x^2+c^2})-1$, and $y=\\log_2(x+x_{\\text{off}})$, where $c=50$ and $x_{\\text{off}}=50$.", fig.height = 5, fig.small = TRUE----
par(mai = c(0.8, 0.6, 0.01, 0.01))
x = seq(-70.5, 170.5, by=1)
cols = c("black", "blue", "darkgrey")
xoff = cc = 50
ymat = cbind(log2.na(x), log2( (x+sqrt(x^2+cc^2))/2 ), log2.na(x+xoff))
ylim = range(ymat, na.rm=TRUE)
matplot(x, ymat, 
        lty=c(1,1,2), col=cols, lwd=3, type="l", ylim=ylim, ylab="")
abline(v = 0, col = "#80808080", lty = 2)
legend(x = par("usr")[2], y = par("usr")[3], 
       legend = c(expression(y = log[2](x)), expression(y = glog[2](x,c)), expression(y = log[2](x+x[off]))), 
       fill=cols, density = c(NA, NA, 50), xjust=1.1, yjust=-0.1)

## ----lymbox, fig.cap = "Boxplot of A values of array CLL-13 stratified by within-sector row. The features in rows 22 and 23 are all very dim."----
colours = hsv(seq(0,1, length = nsr), 0.6, 1)
j = "CLL-13"
boxplot(A[, j] ~ arrayGeometry$spotrow, col = colours, main = j, ylab = "A", xlab = "spotrow")

## ----lymquscp, fig.cap = "MA-plot of the data from array `r j`. Dots coloured in orange are from rows 22 and 23. A possible explanation may be as follows (although I do not know for sure that this is the right explanation): The PCR product (cDNA) that is spotted on these arrays is put on by a print head that sucks cDNA out of microtitre plates and deposits them in spots one after another, row by row. If the content of one plate is faulty, this results in a set of subsequent spots that are faulty. Because the 16 print-tip sectors are spotted in parallel, this affects all sectors in the same way.", fig.small = TRUE----
plot(A[,j], M[,j], pch = 16, cex = 0.3,
  col = ifelse(arrayGeometry$spotrow%in%(22:23), "orange", "black"))
abline(h = 0, col = "blue")

## ----sessionInfo--------------------------------------------------------------
sessionInfo()

.onAttach <- function(libname, pkgname) {
   addVigs2WinMenu("vsn")
}

.onLoad <- function(libname, pkgname) {
  ## register vsn as a normalization method with the affy package
  if(!("vsn" %in% normalize.AffyBatch.methods()))
    upDate.normalize.AffyBatch.methods(c(normalize.AffyBatch.methods(), "vsn"))
}

##
## Test concordance of maxima of profile likelihood and normal likelihood
##

library("vsn")

n = 1000L
d = 2L
nrstr = 1L
nrpar = 2L*d*nrstr

dat = sagmbSimulateData(n=n, d=d, de=0, nrstrata=nrstr, miss=0, log2scale=TRUE)
fit = vsn2(dat$y, lts.quantile=1)

stopifnot(all(diff(dat$strata)>=0)) ## >1 strata not yet implemented here
v = new("vsnInput", x=dat$y, pstart=array(as.numeric(NA), dim=c(nrstr, d, 2)),
  strata=factor(dat$strata), ordered=TRUE)

nplot = 31
par(mfcol=c(2, nrpar))

for(i in seq_len(nrpar))  {
  pars = matrix(coef(fit), nrow=nrpar, ncol=nplot)
  pars[i,] = coef(fit)[i] +  seq(-1, 1, length=nplot)
  logarg  = ""
  if(i<=d*nrstr) {
    xlab = substitute(a[k], list(k=i))
  } else {
    xlab = substitute(b[k], list(k=i-nrpar/2))
  }
  
  for(what in 1:2){
    ll = switch(what,
      logLik(v, pars),                             ## without reference
      logLik(v, pars, mu=fit@mu, sigsq=fit@sigsq)) ## with reference

    plot(pars[i,], ll[1, ], type="b",
         xlab = xlab, log=logarg, main=round(ll[1, (ncol(ll)+1)/2], 1),
         ylab = expression(-log(L)))
    abline(v=coefficients(fit)[i], col="red")
  } ## for what
}

cat("PLL=", logLik(v, matrix(coef(fit), ncol=1)),
   "\n LL=", logLik(v, matrix(coef(fit), ncol=1),  mu=fit@mu, sigsq=fit@sigsq), "\n")

## See further below for class definitions.
## First come the validity functions, because they are needed for that

validScalarNumericSlot = function(ob, nm, min=0, max=+Inf) {
  ## S4 already makes sure s is numeric 
  s = slot(ob, nm)
  if((length(s)!=1)||any(is.na(s))||(s<min)||(s>max))
    return(sprintf("'%s' must be a numeric vector of length 1 with values between %g and %g.", nm, min, max))
  TRUE
}

validScalarDoubleListElt = function(ob, nm, min=0, max=+Inf) {
  s = ob[[nm]]
  if((!is.double(s))||(length(s)!=1)||any(is.na(s))||(s<min)||(s>max))
    return(sprintf("'%s' must be a numeric double precision vector of length 1 with values between %g and %g.", nm, min, max))
  TRUE
}

validScalarIntListElt = function(ob, nm, min=0L, max=.Machine$integer.max) {
  s = ob[[nm]]
  if((!is.integer(s))||(length(s)!=1)||any(is.na(s))||(s<min)||(s>max))
    return(sprintf("'%s' must be an integer vector of length 1 with values between %d and %d.", nm, min, max))
  TRUE
}

validLogical = function(ob, nm) {
  s = slot(ob, nm)
  if((length(s)!=1)||any(is.na(s)))
    return(sprintf("'%s' must be a logical of length one and not be NA.", nm))
  TRUE
}

equalOrZero = function(i, j) ((i==j)||(i==0))

compulsoryElements = c("factr", "pgtol", "maxit", "trace", "cvg.niter", "cvg.eps")

calibCharToInt = function(x)
  switch(EXPR=x, affine=0L, none=1L, stop(sprintf("Invalid value '%s' of 'calib'.", x)))


##--------------------------------------------------
## validity method for 'vsnInput'
##--------------------------------------------------
validityVsnInput = function(object){
  if(any(is.nan(object@x)))
    return("The values in 'x' must be finite numeric or NA; please no NaN.")
  
  r = validScalarNumericSlot(object, "subsample", min=0, max=nrow(object))
  if(!identical(r, TRUE)) return(r)

  r = validScalarNumericSlot(object, "lts.quantile", min=0.5, max=1)
  if(!identical(r, TRUE)) return(r)

  r = validLogical(object, "verbose")
  if(!identical(r, TRUE)) return(r)

  if( (!is.character(object@calib)) ||
      (length(object@calib)!=1L) ||
      (!(object@calib)%in%c("affine","none")) ) return("Invalid slot 'calib'.")

  r = validLogical(object, "ordered")
  if(!identical(r, TRUE)) return(r)

  if(!equalOrZero(length(object@strata), nrow(object@x)))
    return("'length(strata)' must match 'nrow(x)'.")

  if(any(is.na(object@strata)))
    return("'strata' must not contain NA values.")

  if(!is.numeric(object@pstart)||length(dim(object@pstart))!=3)
    return("'pstart' must be a 3D array.")
  
  if(!all(dim(object@pstart)==c(nlevels(object@strata),
                                switch(EXPR=object@calib, affine=ncol(object@x), none=1),
                                2)))
    return("Invalid dimensions of 'pstart'.")

  if(!is.list(object@optimpar)||!identical(names(object@optimpar), optimparNames))
    return(paste("'optimpar' must be a list with elements ",
                 paste("'", compulsoryElements, "'", collapse=", ", sep=""), ".", sep=""))

  r = validScalarDoubleListElt(object@optimpar, "factr")
  if(!identical(r, TRUE)) return(r)
  
  r = validScalarDoubleListElt(object@optimpar, "pgtol")
  if(!identical(r, TRUE)) return(r)

  r = validScalarIntListElt(object@optimpar, "maxit", min=1L)
  if(!identical(r, TRUE)) return(r)

  r = validScalarIntListElt(object@optimpar, "trace", min=0L, max=6L)
  if(!identical(r, TRUE)) return(r)

  r = validScalarIntListElt(object@optimpar, "cvg.niter", min=1L)
  if(!identical(r, TRUE)) return(r)

  r = validScalarDoubleListElt(object@optimpar, "cvg.eps")
  if(!identical(r, TRUE)) return(r)

  return(TRUE)
}


##--------------------------------------------------
## validity method for 'vsn'
##--------------------------------------------------
validityVsn = function(object){
  if (any(is.na(object@coefficients)) || (length(dim(object@coefficients))!=3))
    return("'coefficients' must be a 3D array and not contain NA values.")

  if (dim(object@coefficients)[3L] != 2L)
    return("'dim(coefficients)[3]' must be equal to 2.")
  
  if (length(object@sigsq) != 1L)
    return("'sigsq' must be of length 1.")

  if (length(object@hoffset) != dim(object@coefficients)[1L])
    return("'length(hoffset)' and 'dim(coefficients)[1]' must match.")

  if (!equalOrZero(length(object@strata), length(object@mu)))
    return("'length(strata)' and 'length(mu)' must match.")

  if (nlevels(object@strata) != dim(object@coefficients)[1L])
    return("'nlevels(strata)' and 'dim(coefficients)[1]' must match.")

  ## This does not always have to be the case, see e.g. assignment to 'f8' in the vignette
  ## if (nrow(object@hx) != length(object@mu))
  ##   return("'nrow(hx)' and 'length(mu)' must match.")

  switch(object@calib,
         affine = if(!equalOrZero(ncol(object@hx), dim(object@coefficients)[2]))
           return("'ncol(hx)' and 'dim(coefficients)[2]' must match."),
         none = if(!identical(dim(object@coefficients), c(1L, 1L, 2L)))
           return("'dim(object@coefficients)' must be 'c(1,1,2)'."),
         return("Invalid 'calib'."))

  if (!((length(object@lbfgsb)==1L) && (is.integer(object@lbfgsb))))
    return("'lbfgsb' must be an integer of length 1.")
  
  return(TRUE)
}

##------------------------------------------------------------------------------------
## Class vsn
##------------------------------------------------------------------------------------
setClass("vsn",
  representation(
    coefficients = "array", ## 3D array: nrstrata * d * 2, with 2 parameters
                            ## for each stratum and array.
    strata = "factor", 
    mu = "numeric",
    sigsq = "numeric",
    hx = "matrix",
    lbfgsb = "integer",
    hoffset = "numeric",
    calib = "character"),
         
  prototype(
    coefficients = array(0, dim=c(1L, 0L, 2L)),
    strata = factor(integer(0L), levels="all"),
    mu = numeric(0L),
    sigsq = NA_real_,
    hx = matrix(0, nrow=0L, ncol=0L),
    lbfgsb = NA_integer_,
    hoffset = NA_real_,
    calib = "affine"),
         
  validity = validityVsn
)

##------------------------------------------------------------
## Class vsnInput
##------------------------------------------------------------
setClass("vsnInput",
  representation(
    x  = "matrix",     ## The n*d data matrix
                 
    reference = "vsn", ## A result from a previous fit (for reference normalization)
                 
    strata = "factor", ## Factor of length n, aligned with rows of x.
                       ## The code also recognizes a special case:
                       ## If 'strata' is of length 0, this
                       ##   is a compact way of representing the fact that there is
                       ##   only one stratum, i.e. this is equivalent to 'strata'
                       ##   of length n with all the same values.
                 
    ordered = "logical",  ## Have the rows of x already been sorted so that the
                          ## levels of 'strata' are consecutive (this is only a
                          ## non-trivial condition if there is more than one stratum.
                 
    lts.quantile = "numeric",
    subsample = "integer",
    verbose = "logical",
    calib = "character",
    pstart = "array",     ## Start parameters: see comment on slot 'coefficients'
                          ## in definition of class 'vsn'
    optimpar  = "list"),     ## See below: optimparnames
         
  prototype(
    x = matrix(as.numeric(NA), nrow=0L, ncol=0L),
    reference = new("vsn"),
    strata = factor(integer(0L), levels="all"),
    ordered = FALSE,
    lts.quantile = 1,
    subsample = 0L,
    verbose = TRUE,
    calib = "affine",
    pstart = array(as.numeric(NA), dim=c(1L,0L,2L)),
    optimpar = list(factr=5e7, pgtol=2e-4, 
                 maxit=60000L, trace=0L, cvg.niter=7L, cvg.eps=0)),
         
  validity = validityVsnInput)
                  

optimparNames = c("factr", "pgtol", "maxit", "trace", "cvg.niter", "cvg.eps")

##------------------------------------------------------
## Test whether the likelihood estimator recovers the
## correct parameters, and plot the likelihood landscape
##--------------------------------------------------

library("vsn")
options(error=recover)
graphics.off()
## set.seed(0xbeeb)

## Generate data
n = 500*(2^(1:9))
dat = sagmbSimulateData(n=n[length(n)], d=1, de=0, nrstrata=1, miss=0, log2scale=TRUE)
ref = new("vsn", mu=dat$mu, sigsq=dat$sigsq, coefficients=dat$coefficients, hoffset=rep(NA_real_, dim(dat$coefficients)[1]))
vin = new("vsnInput", x=dat$y, pstart=array(as.numeric(NA), dim=c(1,1,2)), lts.quantile = 1)

## series of fits with ascending amounts of data
fitpar = sapply(n, function(k) {
  subs = 1:k
  fit = vsn2(dat$y[subs,], reference=ref[subs,], lts.quantile=1, verbose=FALSE)
  coef(fit)
})

graphics.off()

cat("----------------------------------------------------------------------------------\n",
    " Test 1:\n",
    " Do the estimated parameters converge to the true ones when the data is increased?\n",
    "----------------------------------------------------------------------------------\n\n", sep="")
x11(height=6, width=5)
par(mfrow=c(nrow(fitpar), 1))
for(i in 1:nrow(fitpar)) {
  target = if(i<=2) dat$coefficients[1,1,i] else 0
  plot(sqrt(n), fitpar[i,], pch=20, xlab=expression(sqrt(n)), main="Test 1", 
       ylab=letters[i], ylim=range(c(fitpar[i,], target)))
  abline(h=target, col="blue")
}

## pedestrian calculation of logLik
myLLfun = function(object, p, mu, sigsq) {
  res = matrix(as.numeric(NA), nrow=1, ncol=ncol(p))
  stopifnot(nrow(p)==2)
  for(j in 1:ncol(p))  {
    fb = scalingFactorTransformation(p[2,j])
    Y = p[1,j]+(object@x*fb)
    h = asinh(Y)
    scale = nrow(object)/2*log(2*pi*sigsq)
    residuals = sum((h-mu)^2/(2*sigsq))
    jacobi = -nrow(object)*log(fb) + 0.5*sum(log(1+Y^2))
    res[, j] = -(scale+residuals+jacobi)
  }
  return(res)
}

cat("----------------------------------------------------------------------------------\n",
    " Test 2:\n",
    " How does the likelihood landscape look around the true parameters?\n",
    "----------------------------------------------------------------------------------\n\n", sep="")
dat = sagmbSimulateData(n=16000, d=1, de=0, nrstrata=1, miss=0, log2scale=TRUE)
ref = new("vsn", mu=dat$mu, sigsq=dat$sigsq, coefficients=dat$coefficients, hoffset=rep(NA_real_, dim(dat$coefficients)[1]))
vin = new("vsnInput", x=dat$y, pstart=array(as.numeric(NA), dim=c(1,1,2)), lts.quantile = 1)

ex = c(.2, .2)
x11(xpos=400)
lp1 = plotVsnLogLik(vin, dat$coefficients, mu=ref@mu, sigsq=ref@sigsq, expand=ex, main="Test 2")

cat("----------------------------------------------------------------------------------\n",
    " Test 3:\n",
    " Does the likelihood landscape around the true parameters look the same when\n",
    " computed using 'myLLfun'?\n",
    "----------------------------------------------------------------------------------\n\n", sep="")
x11(xpos=800)
lp2 = plotVsnLogLik(vin, dat$coefficients, mu=ref@mu, sigsq=ref@sigsq, expand=ex, fun=myLLfun, main="Test 3")
if(max(abs(lp1$logLik-lp2$logLik))>1e-7) warning("lp1 and lp2 are different!")

wm = which.max(lp1$logLik)
cat(sprintf("True parameters   %11g %11g\n", dat$coefficients[1], dat$coefficients[2]))
cat(sprintf("Maximum of logLik %11g %11g\n\n", lp1[wm,1], lp1[wm,2]))


cat("----------------------------------------------------------------------------------\n",
    " Test 4:\n",
    " What happen if noise is only multiplicative (no profiling)?\n",
    "----------------------------------------------------------------------------------\n\n", sep="")
n = 32000L
mu = runif(n)*2+4
sigsq = 0.01
y  = exp(mu+matrix(rnorm(2L*length(mu), sd=sqrt(sigsq)), ncol=2L))

stopifnot(identical(scalingFactorTransformation(1), exp(1)))

ref = new("vsn", mu=mu, sigsq=sigsq,
          coefficients = array(c(10, -log(2)), dim=c(1,1,2)),
          hoffset = rep(NA_real_, 1))

v1 = vsn2(x=y, ref=ref, lts.quantile=1)
hy = predict(v1, newdata=y)

x11(width=7, height=4); par(mfrow=c(1,2))
plot(log2(y), hy, pch=".", main="Test 4")
abline(a=0, b=1, col="orange")

cat("----------------------------------------------------------------------------------\n",
    " Test 5:\n",
    " What happen if noise is only multiplicative (with profiling)?\n",
    "----------------------------------------------------------------------------------\n\n", sep="")
v2 = vsn2(x=y, lts.quantile=1)

hy = predict(v2, newdata=y)
plot(log2(y), hy, pch=".", main="Test 5")
abline(a=0, b=1, col="orange")

##----------------------------------------------------------------------------
## mu as in the statistical model - expectation of transformed intensities 
## generate sinh(mu) according to model of Newton et al.
## shape parameter a=1, scale theta=1
## Since vsn 2.0, results are returned on log2 scale, rather than natural log.
##----------------------------------------------------------------------------
sagmbSimulateData <- function(n=8064, d=2, de=0, up=0.5, nrstrata=1, miss=0, log2scale=FALSE) {
  stopifnot(is.numeric(n),  length(n)==1, n>=1)
  stopifnot(is.numeric(d),  length(d)==1, d>=1) 
  stopifnot(is.numeric(de), length(de)==1, de>=0, de<=1)
  stopifnot(is.numeric(up), length(up)==1, up>=0, up<=1)
  stopifnot(is.numeric(nrstrata), length(nrstrata)==1)

  sigsq <- 0.04
  mu <- asinh(1/rgamma(n, shape=1, scale=1))

  ##------------------------------------------------------------
  ## the calibration parameters: 
  ##------------------------------------------------------------
  coefficients = array(c(runif(nrstrata*d, min=-2, max=+2),  ## offsets
                         runif(nrstrata*d, min=-1, max=+1)), ## factors
                         dim=c(nrstrata, d, 2L))

  ##------------------------------------------------------------
  ## generate simulated data:
  ##  hy = asinh(f(b)*y+a)   <=>  y = (sinh(hy)-a)/f(b)
  ##------------------------------------------------------------
  is.de  <- ((runif(n)<de ) & (mu > quantile(mu, 0.3)))
  hy     <- matrix(as.numeric(NA), nrow=n, ncol=d)
  hy[,1] <- mu + rnorm(n, sd=sqrt(sigsq))    ## array 1 is a reference
  for (j in seq_len(d)[-1]) {
    s      <- 2 * as.numeric(runif(n)<up) - 1
    hy[,j] <- mu + as.numeric(is.de)*s*runif(n, min=0, max=2) + rnorm(n, sd=sqrt(sigsq))
  }
  strata <- as.integer(ceiling(runif(n)*nrstrata))
  offs   <- coefficients[strata,,1]
  facs   <- coefficients[strata,,2]
  stopifnot(all(dim(facs)==dim(hy)), all(dim(offs)==dim(hy)))

  y <- (sinh(hy)-offs)/scalingFactorTransformation(facs)
  if(miss>0)
    y[sample(length(y), length(y)*miss)] = as.numeric(NA)

  if(log2scale)
    hy = hy/log(2)
  
  return(list(y=y, hy=hy, mu=mu, sigsq=sigsq, coefficients=coefficients, is.de=is.de, strata=strata))
}

## assess
sagmbAssess <- function(h1, sim) {
  stopifnot(all(c("y", "hy", "is.de") %in% names(sim)))
  h1    <- unname(h1)
  h2    <- sim$hy
  is.de <- sim$is.de

  stopifnot(is.matrix(h1), is.matrix(h2), is.logical(is.de),
            identical(is.na(h1), is.na(sim$y)), !any(is.na(h2)))
  
  n <- nrow(h1)
  d <- ncol(h1)
  if((nrow(h2)!=n) || (ncol(h2)!=d))
    stop(paste("'h1' and 'h2' should be matrices of the same size, but have ",
         n, "x", d, " and ", nrow(h2), "x", ncol(h2), sep=""))
  stopifnot(length(is.de)==n)

  dh1 <- h1-rowMeans(h1, na.rm=TRUE)
  dh2 <- h2-rowMeans(h2)

  dh   <- dh1[!is.de,] - dh2[!is.de,]
  sqrt(mean(dh^2, na.rm=TRUE))
}

##----------------------------------------------------------------------
## Robust calibration and variance stabilization
##   revised version vsn2
## (C) Wolfgang Huber <huber@ebi.ac.uk> 2002-2007
## contributions from Markus Ruschhaupt, Dennis Kostka, David Kreil
##
## The computations are organised as a set of nested function calls.
## The innermost is vsnML, which calls the C code.
## The next one is vsnLTS, which does the robust modification (least
##   trimmed sum of squares, LTS) of the maximum likelihood (ML) estimator.
## The next one is vsnStrata, which if necessary reorders the data so
##   that features from the same stratum are adjacent.
## The next one is vsnSample, which allows for sampling.
## The next one is vsnMatrix, a function that accepts a matrix with the
##   data and the various user-defined parameters
## The outmost ones are various S4 methods that call vsnMatrix.
##----------------------------------------------------------------------

##----------------------------------------------------------------------------
## ML estimator
##-----------------------------------------------------------------------------
vsnML = function(v) {

  p = as.vector(v@pstart)
  istrat = calcistrat(v) ## pointers to the starts of strata

  o = .Call("vsn2_optim",
    v@x,
    p,
    istrat,
    v@reference@mu,
    v@reference@sigsq,
    v@optimpar,
    calibCharToInt(v@calib),
    PACKAGE="vsn")

  rv = new("vsn",
    coefficients = o$coefficients, 
    strata = v@strata,
    mu = o$mu,
    sigsq = o$sigsq,
    hx = matrix(NA_real_, nrow=nrow(v@x), ncol=0),
    lbfgsb = o$fail,
    hoffset = rep(NA_real_, nlevels(v@strata)),
    calib = v@calib)

  if (o$fail!=0L) {
    nrp = if(length(p)<=6) length(p) else 6L
    msg = paste("** This is a diagnostic message on the performance of the optimizer,\n",
                "** it need not indicate a relevant problem:\n",
                sprintf("fail=%d\npstart[1:%d]=", o$fail, nrp),
                paste(signif(p[1:nrp], 4), collapse=", "),
                sprintf("\n  coef[1:%d]=", nrp),
                paste(signif(coefficients(rv)[1:nrp], 4), collapse=", "), "\n", sep="")
    ## warning(msg)
  }
  
  return(rv)
}

##------------------------------------------------------------
## istrat[j] is the starting position for stratum j
## i.e. istrat[j]...istrat[j+1] are the elements of ysel that
## belong to stratum j (using C indexing convention,
## i.e. starting at 0).
## Note: the counting over the different samples is folded into j,
## i.e., if there are 4 strata on the array and 3 samples, then
## j runs from 1:12
##------------------------------------------------------------
calcistrat = function(vp) {
  nrs = nlevels(vp@strata)

  switch(vp@calib,
    affine = {
      if(length(vp@strata)>0) {
        stopifnot(vp@ordered)
        istr = which(!duplicated(vp@strata))-1L
      } else {
        istr = 0L
      }
      stopifnot(length(istr)==nrs)

      istrat = integer(ncol(vp@x)*nrs+1L)
      for(i in 0:(ncol(vp@x)-1L))
        istrat[i*nrs + seq_len(nrs)] = i*nrow(vp@x) + istr

      istrat[length(istrat)] = nrow(vp@x)*ncol(vp@x)  ## end point
    },
         
    none = {
      if(nrs>1) stop("There must not be more than 1 stratum if 'calib' is 'none'")
      istrat = c(0L, nrow(vp@x)*ncol(vp@x))
    },
         
    stop(sprintf("Invalid value for 'vp@calib': %s", vp@calib))
  ) ## end of switch
  
  return(istrat)
}

##-------------------------------------------------------------------------
## LTS robust modification of the ML estimator
##-------------------------------------------------------------------------
vsnLTS = function(v) {

  ## for calculating a convergence criterion: earlier result
  oldhy   = Inf
  ## the number of iterations that have already met the convergence criterion
  cvgcCnt = 0
  
  ## integer version of "v@strata"
  intstrata = if(length(v@strata)==0L) rep(1L, nrow(v@x)) else as.integer(v@strata)
  facstrata = factor(intstrata, levels=paste(1:nlevels(v@strata)))
  stopifnot(!any(is.na(facstrata)))
    
  for(iter in seq_len(v@optimpar$cvg.niter)) {

    sv  = if(iter==1) v else v[whsel, ]
    rsv = vsnML(sv)

    ## if LTS.quantile is 1, then the following stuff is not necessary
    if (isSmall(v@lts.quantile-1))
      break
       
    ## apply to all data
    hy = vsn2trsf(x=v@x, p=coefficients(rsv), strata=intstrata, calib=v@calib)
    v@pstart = coefficients(rsv)

    ## Calculate residuals
    if(length(v@reference@mu)>0) {
      ## with reference:
      hmean = v@reference@mu
    } else {
      ## without reference:
      hmean = rowMeans(hy, na.rm=TRUE) 
      ## double check with what was computed in vsnML:
      if(iter==1L) {
        if(rsv@lbfgsb==0L) stopifnot(isSmall(rsv@mu-hmean))
      } else {
        if(rsv@lbfgsb==0L) stopifnot(isSmall(rsv@mu-hmean[whsel]))
        ## and create rsv@mu with mu of the right length (NA for the 'outliers' not in whsel)
        tmp = rep(NA_real_, nrow(v))
        tmp[whsel] = rsv@mu
        rsv@mu = tmp
      }
    }
    
    ## sum of squared residuals for each row
    rvar  = rowSums((hy-hmean)^2)

    ## select those data points whose rvar is within the quantile; do this separately
    ## within each stratum, and also within 5 slices defined by hmean
    ## (see the SAGMB 2003 paper for details)
    nrslice = 5L
    facslice = cut(rank(hmean, na.last=TRUE), breaks=nrslice)
    slice = as.integer(facslice)
    
    grquantile = tapply(rvar, list(facslice, facstrata), quantile, probs=v@lts.quantile, na.rm=TRUE)

    ## Only use those datapoints with residuals less than grquantile,
    ##    but use all datapoints for slice 1 (the lowest intensity spots)
    whsel = which((rvar <= grquantile[ cbind(slice, intstrata) ]) |
                  (slice == 1L))
    
    ## Convergence check
    ## after a suggestion from David Kreil
    if(v@optimpar$cvg.eps>0) {
      cvgc    = max(abs(hy - oldhy), na.rm=TRUE)
      cvgcCnt = if(cvgc<v@optimpar$cvg.eps) (cvgcCnt+1) else 0 
      if(cvgcCnt>=3)
        break
      oldhy = hy
    }

  } ## end of for-loop
  
  return(rsv)
}

##----------------------------------------------------------------------------------
## vsnColumnByColumn
##----------------------------------------------------------------------------------
vsnColumnByColumn = function(v) {
  rlv = vsnLTS(v[,1])
  
  d = dim(coefficients(rlv))
  n = ncol(v)
  stopifnot(length(d)==3, identical(d[2], 1L))
  cf = array(NA_real_, dim=c(d[1], n, d[3]))
  cf[,1,] = coefficients(rlv)
  
  for(j in seq_len(n)[-1]) {
    cf[,j,] = coefficients(vsnLTS(v[,j]))
  }
  
  return(new("vsn",
             coefficients = cf, 
             strata = v@strata,
             mu = v@reference@mu,
             sigsq = v@reference@sigsq,
             hoffset = rep(NA_real_, nlevels(v@strata)),
             calib = v@calib))
}

##----------------------------------------------------------------------------------
## vsnStrata: if necessary,
## reorder the rows of the matrix so that each stratum sits in a contiguous block
##----------------------------------------------------------------------------------
vsnStrata = function(v) {

  ord = NULL
  if(nlevels(v@strata)>1) {
    ord = order(v@strata)
    ## reorder the rows of the matrix so that each stratum sits in a contiguous block
    v = v[ord,]
  }
  v@ordered = TRUE

  res = if(length(v@reference@mu)>0L) {
    vsnColumnByColumn(v)
  } else {
    vsnLTS(v)
  }

  ## reverse the ordering
  res@mu[ord] = res@mu
  
  return(res)
} 

##----------------------------------------------------------------------------------
## vsnSample: allows for (balanced) subsetting
##----------------------------------------------------------------------------------
vsnSample = function(v) {

  if( (v@subsample>0L) && (length(v@reference@mu)>0L) )
    stop("\n\nThe 'subsample' and 'normalization to reference' options cannot be mixed. ",
         "'normalization to reference' needs to use the same subsampling as the one ",
         "used for creating the reference. If you want to use 'normalization to reference', ",
         "please call this function without the 'subsample' argument.\n\n")

  wh = NULL
  
  if(v@subsample>0L) {
    if((length(v@strata)>0L) && (nlevels(v@strata)>1L)) { 
      sp = split(seq(along=v@strata), v@strata)
      wh = unlist(lapply(sp, sample, size=v@subsample))
    } else {
      wh = sample(nrow(v), size=v@subsample)
    }
  }

  if(length(v@reference@mu)>0L) {
    wh = which(!is.na(v@reference@mu))
  }

  ## remove those rows that are all NA
  allNA = (rowSums(is.na(v@x))==ncol(v@x))
  numNA = sum(allNA)
  if(numNA>0) {
    wh = if(is.null(wh)) which(!allNA) else setdiff(wh, which(allNA))
    warning(sprintf("%d rows were removed since they contained only NA elements.", numNA))
  }
  
  if(!is.null(wh)){
    
    res = vsnStrata(v[wh, ])

    ## put back the results from subsampling 
    newmu = rep(NA_real_, nrow(v))
    newmu[wh] = res@mu
    res@mu = newmu
    
  } else {

    res = vsnStrata(v)
  }
  
  return(res)
}

##----------------------------------------------------------------------
## vsnMatrix
##----------------------------------------------------------------------
vsnMatrix = function(x,
             reference,
             strata,
             lts.quantile = 0.9,
             subsample    = 0L,
             verbose      = interactive(),
             returnData   = TRUE,
             calib        = "affine",
             pstart,
             minDataPointsPerStratum = 42L,
             optimpar   = list(),
             defaultpar = list(factr=5e7, pgtol=2e-4, maxit=60000L,
                               trace=0L, cvg.niter=7L, cvg.eps=0))
{
  storage.mode(x) = "double"
  storage.mode(subsample) = "integer"
  
  strata = if(missing(strata)){ 
    factor(integer(0), levels="all")
  } else {
    int2factor(strata)
  }
  
  stratasplit = if(length(strata)>0L) split(seq_len(nrow(x)), strata) else list(all=seq_len(nrow(x)))
  if(!(identical(names(stratasplit), levels(strata)) &&
       all(listLen(stratasplit) >= minDataPointsPerStratum))) {
    msg = if(length(stratasplit)>1){
      ## more than one stratum
      paste("vsnMatrix: this function is worried that for some strata, the number of corresponding rows in the data matrix is too small for reliable estimation of the vsn transformation parameters. Strata sizes are:", paste(sort(unique(listLen(stratasplit))), collapse=", "), ". Consider reducing the number of strata.")
    } else {
      ## one stratum
      sprintf("This function is worried that the number of rows in the data matrix, %d, is too small for reliable estimation of the vsn transformation parameters. If you think that your data really have more rows, then please check whether there is a data transmission problem (e.g., matrix transposition?).", nrow(x))
    }
    msg = paste(msg, "Otherwise, if you are sure about what you are doing, you can change the parameter 'minDataPointsPerStratum' in order to reduce the minimal number of rows acceptable for proceeding.")
    stop(msg)
  }
    
  if(missing(pstart))
    pstart = pstartHeuristic(x, stratasplit, calib)
  
  if(missing(reference)) {
    if(ncol(x)<=1L)
      stop("'x' needs to have 2 or more columns if no 'reference' is specified.")
    reference = new("vsn")
  } else {
    if(nrow(reference)!=nrow(x))
      stop("'nrow(reference)' must be equal to 'nrow(x)'.")
    if(nrow(reference)!=length(reference@mu))
      stop(sprintf("The slot 'reference@mu' has length %1$d, but expected is n=%1$d", nrow(reference)))
  }

  if(!(is.list(optimpar)&&all(names(optimpar)%in%optimparNames)&&!any(duplicated(names(optimpar)))))
    stop(paste("Names of elements of 'optimpar' must be from: ",
               paste("'", optimparNames, "'", collapse=", ", sep=""), ".", sep=""))
  opar = defaultpar
  opar[names(optimpar)] = optimpar
  
  v = new("vsnInput",
    x = x,
    strata = strata,
    pstart = pstart,
    reference = reference,
    lts.quantile = lts.quantile,
    optimpar = opar,
    subsample = subsample,
    verbose = verbose,
    calib = calib,
    ordered = FALSE)
  
  ## Print welcome message
  if (verbose)
    message("vsn2: ", nrow(x), " x ", ncol(x), " matrix (", nlevels(strata), " strat",
        ifelse(nlevels(strata)==1L, "um", "a"), "). ")

  ## Here, the actual work is done.
  res = vsnSample(v)

  ## hoffset: compute from average scale factors between arrays, but separately for the different strata.
  ## coefficients 'cof' from are taken from the reference, if available.
  cof = coefficients( if(nrow(reference)==0L) res else reference )
  res@hoffset = log2(2*scalingFactorTransformation(rowMeans(cof[,,2L,drop=FALSE])))
  
  ## If necessary, calculate the data matrix transformed according to 'coefficients'
  if(returnData) {
    res@strata = strata
    res@hx = vsn2trsf(x = x,
      p = coefficients(res),
      strata = as.integer(res@strata),
      hoffset = res@hoffset,
      calib = calib)
  }
  
  stopifnot(validObject(res))

  if(verbose) {
    message("Please use 'meanSdPlot' to verify the fit.")
  }
  return(res)
}

##---------------------------------------------------------------------
## The glog transformation
##--------------------------------------------------------------------
vsn2trsf = function(x, p, strata = numeric(0L), hoffset = NULL, calib) {
  if (!is.matrix(x) || !is.numeric(x))
    stop("'x' must be a numeric matrix.\n")
  
  if (!is.array(p) || !is.numeric(p)) stop("'p' must be a numeric array.\n")
  if (any(is.na(p))) stop("'p' must not contain NAs.\n")

  if(length(strata)==0L) {
    strata = rep(1L, nrow(x))
    nrstrata = 1L
  } else {
    if(! (is.integer(strata) && is.vector(strata) &&
         identical(length(strata), nrow(x)) &&
         (!any(is.na(strata)))) )
      stop("'strata' must be an integer vector of length nrow(x) with no NAs.")
    nrstrata = max(strata)
  }

  if(nrstrata==1L && length(dim(p))==2L)
    dim(p) = c(1L, dim(p))

  d2 = switch(calib,
    affine= ncol(x),
    none = 1,
    stop(sprintf("Invalid value of 'calib': %s", calib)))

  if(length(dim(p))!=3L ||
     dim(p)[1L] != nrstrata ||
     dim(p)[2L] != d2 ||
     dim(p)[3L] != 2L)
    stop("'p' has wrong dimensions.")
  
  hx = .Call("vsn2_trsf",
    x,
    as.vector(p),
    strata,
    calibCharToInt(calib),
    PACKAGE="vsn")
  
  dimnames(hx) = dimnames(x)
  
  ## see the 'value' section of the man page of 'vsn2'
  if(!is.null(hoffset)) {
    stopifnot(length(hoffset)==nrstrata)
    hx = hx/log(2) - hoffset[strata]
  }
  return(hx)
}

##-----------------------------------------------------------------------------
## The scaling factor transformation (function "f" in the vignette
## 'likelihoodcomputations.Rnw')
##------------------------------------------------------------------------------
scalingFactorTransformation = function(b) {
  .Call("vsn2_scalingFactorTransformation", b, PACKAGE="vsn")
}

##----------------------------------------------------------
## row-wise variances of a matrix (slightly more efficient
##  than genefilter:rowVars since can pass 'mean')
##-----------------------------------------------------------
rowV = function(x, mean, ...) {
  sqr     = function(x)  x*x
  n       = rowSums(!is.na(x))
  n[n<1]  = NA
  if(missing(mean))
    mean=rowMeans(x, ...)
  return(rowSums(sqr(x-mean), ...)/(n-1))
}

##--------------------------------------------------------------
## A heuristic to set the start parameters for offset and scale.
##--------------------------------------------------------------
pstartHeuristic = function(x, sp, calib) {
  
  d2 = switch(calib,
    affine= ncol(x),
    none = 1,
    stop(sprintf("Invalid value of 'calib': %s", calib)))

  pstart = array(0, dim=c(length(sp), d2, 2))
  pstart[,,2] = 1
  return(pstart)
}

##-------------------- 
## integer to factor
##-------------------- 
int2factor = function(strata) {
  if(is.factor(strata))
    return(strata)
  if(!is.integer(strata))
    stop("'strata' needs to be an integer vector.")
  ssu = sort(unique(strata))
  if(!identical(ssu, seq(along=ssu)))
    stop("'strata' needs to be an integer vector whose values cover ",
         "the range of  1...n.")
  factor(strata, levels=ssu)
}

##-------------------------------------------------------
## check if all elements of a vector are close to 0
##--------------------------------------------------------
isSmall = function(x, tol=sqrt(.Machine$double.eps)) (max(abs(x))<tol)

## ----------------------------------------------------------------------
## This file contains the steps that were used 
## 1.+2. to prepare the example datasets "lymphoma" and "kidney"
##       of the package in 2002
## 3.+4. update them from exprSet to ExpressionSet on 9 Feb 2007.
## ----------------------------------------------------------------------
dataout   = "../../data"
lym = file.path(dataout, 'lymphoma.RData')
kid = file.path(dataout, 'kidney.RData')

library("Biobase")

## ------------------------------------------------------------
## 1. create lymphoma
## ------------------------------------------------------------
samples   = read.delim("lymphomasamples.txt", as.is=T)
datain    = "/home/whuber/h/VSN/alizadeh"

## CH1  = Cy3 = green =  reference
## CH2  = Cy5 = red   =  sample of interest

nrspots   = 9216
nrsamples = nrow(samples)
qua       = matrix(NA, nrow=nrspots, ncol=2*nrsamples)
pd        = data.frame(name    = I(character(2*nrsamples)),
                       sample  = I(character(2*nrsamples)))
  
for (i in 1:nrsamples) {
  filename = paste(samples$name[i], 'rex.DAT', sep='')
  dat = read.delim(file.path(datain, filename))
  qua[,2*i-1] = dat$CH1I - dat$CH1B
  qua[,2*i]   = dat$CH2I - dat$CH2B
  pd$name[(2*i-1):(2*i)]    = samples$name[i]
  pd$sample[2*i-1]  = "reference"
  pd$sample[2*i]    = samples$sampleid[i]
}
colnames(qua) = pd$sample

lymphomaPhenoData <- new("AnnotatedDataFrame")
pData(lymphomaPhenoData) <- pd
varLabels(lymphomaPhenoData) <- list(name="Name of the Chip", sample="Sample")

lymphoma = new("ExpressionSet",
    exprs = qua,
    phenoData = lymphomaPhenoData)
    
save(lymphoma, file=lym, compress=TRUE)

## ------------------------------------------------------------
## 2. create kidney
## ------------------------------------------------------------
datain = "/net/herkules/raid4/home/whuber/Kidney2"
thehyb = 90
load(file.path(datain, "squa.Rdata"))

dat = (squa[, c("fg.green", "fg.red"), thehyb]
      -squa[, c("bg.green", "bg.red"), thehyb])
rownames(dat) = NULL
colnames(dat) = c("green", "red")

kidneyPhenoData <- new("AnnotatedDataFrame")
pData(kidneyPhenoData) <- data.frame(channel = c("green", "red"))
varLabels(kidneyPhenoData) <- list(channel="green: 532 nm, dye=Cy3; red: 635 nm, dye=Cy5")

kidney = new("ExpressionSet",
  exprs = dat,
  phenoData = kidneyPhenoData)

save(kidney, file=kid, compress=TRUE)




## ------------------------------------------------------------
## 3. create lymphoma
## ------------------------------------------------------------
load(lym)
pd = pData(lymphoma)
ex = exprs(lymphoma)

pd$dye = factor(paste("Cy", rep(c(3L, 5L), 8), sep=""))

rownames(pd) = colnames(ex) = paste(pd$name, pd$sample, sep=".")

vmd = data.frame(labelDescription=I(c("Array ID", "Sample", "Dye")))
rownames(vmd) = colnames(pd)
ad = new("AnnotatedDataFrame", data=pd, varMetadata=vmd)

lymphoma = new("ExpressionSet", exprs=ex, phenoData=ad)

save(lymphoma, file=lym)


## ------------------------------------------------------------
## 4. update kidney
## ------------------------------------------------------------
load(kid)

pd =  pData(kidney)
ex =  exprs(kidney)
rownames(pd) = colnames(ex)

vmd = data.frame(labelDescription=I("The scanner channel Cy3 or Cy5"))
rownames(vmd) = colnames(pd)
ad = new("AnnotatedDataFrame", data=pd, varMetadata=vmd)
    
kidney = new("ExpressionSet", exprs=ex, phenoData=ad)

save(kidney, file=kid)

justvsn = function(x,  ...) {
    fit = vsn2(x, ...)
    predict(fit, newdata=x, useDataInFit=TRUE)
  }

vsnrma = function(x, ...) {
  fit = vsn2(x, ...)
  exprs(x) = 2^exprs(fit)
  
  ## call RMA to do the probeset summarization,
  ## (with no background correction / normalization, that is already done)
  rv = rma(x, normalize=FALSE, background=FALSE)
  if(!(is(rv, "ExpressionSet") && validObject(rv)))
    stop("Failed to create a valid 'ExpressionSet' object.")
  return(rv)
}

##------------------------------------------------------------
## methods for the generic function 'vsn2'
##------------------------------------------------------------
setMethod("vsn2", "matrix", vsnMatrix)


setMethod("vsn2", "numeric",
   function(x,  reference, strata, ...)
      vsnMatrix(as.matrix(x, ncol=1), reference, strata, ...))


setMethod("vsn2", "ExpressionSet",
   function(x,  reference, strata, ...)
      vsnMatrix(exprs(x),  reference, strata, ...))


setMethod("vsn2", "NChannelSet",
   function(x,  reference, strata, backgroundsubtract=FALSE, 
            foreground=c("R","G"), background=c("Rb", "Gb"), ...) {
     
     y = MatrixFromNChannelSet(x, backgroundsubtract=backgroundsubtract,
                               foreground=foreground, background=background)
     res = vsnMatrix(y, reference, strata, ...)
     attr(res, "ChannelNames") = foreground
     return(res)
   })


setMethod("vsn2", "AffyBatch",
   function(x, reference, strata, subsample, ...) {
     dat = exprs(x)
     if(missing(subsample))
       subsample = if((nrow(dat)>30000L) && missing(reference)) 30000L else 0L
     vsnMatrix(dat, reference, strata, optimpar=list(cvg.niter=4L),
       subsample = subsample, ...)
    })


setMethod("vsn2", "RGList",
   function(x, reference, strata, ...)
      vsn2(as(x, "NChannelSet"), reference, strata, ...))


MatrixFromNChannelSet = function(x, backgroundsubtract, foreground, background) {
  ad = assayData(x)
  if(!all(foreground%in%channelNames(x)))
    stop("One or more elements of 'foreground' are not contained in 'channelNames(x)'.")

  ## list of matrices with the foreground values
  lmat = lapply(foreground, function(k) {
    rv = ad[[k]]
    return(rv)
  })
  ## one wide matrix with all of them next to each other
  y = do.call(cbind, lmat)
  
  if(!(is.logical(backgroundsubtract)&&(length(backgroundsubtract)==1)&&(!is.na(backgroundsubtract))))
    stop("'backgroundsubtract' must be a logical of length 1 and not NA.")
  
  if(backgroundsubtract){
    if(!all(background%in%channelNames(x)))
      stop("One or more elements of 'background' are not contained in 'channelNames(x)'.")
    stopifnot(length(background)==length(foreground))
    lmat = lapply(background, function(k) ad[[k]])
    y = y - do.call(cbind, lmat)
  }

  return(y)
}

##----------------------------------------------------------------
## given vsnInput object (containg data matrix and possibly strata)
## calculate likelihood for parameters p
##----------------------------------------------------------------
checkArgs = function(object, mu, sigsq) {

  if(length(mu)>0) {
    if(length(mu)!=nrow(object))
      stop("length(mu) must be either 0 or nrow(object).")
    if((length(sigsq)!=1) || !is.numeric(sigsq) || !is.finite(sigsq))
      stop("'sigsq' must be a finite scalar numeric not NA.")
  } else {
    if(!is.na(sigsq))
      stop("If length(mu) is 0, sigsq must not be specified.")
  }
}

vsnLogLik = function(object, p, mu=numeric(0), sigsq=as.numeric(NA), calib="affine") {
  
  checkArgs(object, mu, sigsq)
  if(is.vector(p))
    dim(p)=c(length(p), 1)  
    
  res = matrix(as.numeric(NA), nrow=1+nrow(p), ncol=ncol(p))
  istrat = calcistrat(object) 
  for(j in 1:ncol(p))
    res[, j] = .Call("vsn2_point",
         object@x,
         p[,j],
         istrat,
         mu,
         sigsq,
         calibCharToInt(calib),
         PACKAGE="vsn")

  ## invert sign since the above calculates the _negative_ log likelihood
  return(-res)
}

vsnHessian = function(object, p, mu=numeric(0), sigsq=as.numeric(NA), eps=1e-4) {
  checkArgs(object, mu, sigsq)

  if((length(dim(p))!=3)||(any(dim(p)!=c(nlevels(object@strata), ncol(object), 2L))))
    stop("'p' has wrong dimensions.")

  istrat = calcistrat(object)

  np = length(p)
  pp = array(p, dim=c(np, np, 2L))
  dp = ifelse(1:np <= prod(dim(p)[1:2]), eps, p*exp(eps))
  for(j in seq_len(np)) { 
    pp[j, j, 1] = pp[j, j, 1] - dp[j]/2
    pp[j, j, 2] = pp[j, j, 2] + dp[j]/2
  }
  dim(pp) = c(np, np*2L)
  ll = vsnLogLik(object, pp, mu, sigsq)[-1, ]
  dim(ll) = c(np, np, 2L)

  res = matrix(as.numeric(NA), nrow=np, ncol=np)
  for(i in seq_len(np))
    for(j in seq_len(np))
      res[i, j] = (ll[i, j, 2] - ll[i, j, 1])/dp[j]
  
  return(res)
}

setGeneric("vsn2",
  def = function(x, reference, strata, ...) standardGeneric("vsn2"),
  valueClass = "vsn")

setGeneric("predict") ## S3 function imported from package stats
setGeneric("nrow")    ## S3 from base
setGeneric("ncol")   
setGeneric("dim")    


setGeneric("meanSdPlot",
  function(x, ranks = TRUE, xlab = ifelse(ranks, "rank(mean)", "mean"),
           ylab = "sd", pch, plot = TRUE, bins = 50, ...)
   standardGeneric("meanSdPlot"))

## ==========================================================================
## meanSdPlot method for matrix
## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
setMethod("meanSdPlot", signature = "matrix", definition =
    function(x, ranks = TRUE, xlab = ifelse(ranks, "rank(mean)", "mean"),
             ylab = "sd", pch, plot = TRUE, bins = 50, ...) {
      
      stopifnot(is.logical(ranks), length(ranks) == 1, !is.na(ranks))

      n = nrow(x)
      if (n == 0L) {
        warning("In 'meanSdPlot': input matrix 'x' has 0 rows. There is nothing to be done.")
        return()
      }
      if (!missing(pch)) {
        warning("In 'meanSdPlot': 'pch' is ignored.")
      }
      
      px   = rowMeans(x, na.rm = TRUE)
      py   = sqrt(rowV(x, mean = px, na.rm = TRUE))
      rpx  = rank(px, na.last = FALSE, ties.method = "random")
                
      ## run median with centers at dm, 2*dm, 3*dm,... and width 2*dm
      dm        = 0.025
      midpoints = seq(dm, 1-dm, by = dm)
      within    = function(x, x1, x2) { (x >= x1) & (x <= x2) }
      mediwind  = function(mp) median(py[within(rpx/n, mp - 2*dm, mp + 2*dm)], na.rm = TRUE)
      rq.sds    = sapply(midpoints, mediwind)
                
      res = if(ranks) {
        list(rank = midpoints*n, sd = rq.sds, px = rpx, py = py)
      } else {
        list(quantile = quantile(px, probs = midpoints, na.rm = TRUE), sd = rq.sds, px = px, py = py)
      }
      
      fmt = function() function(x) format(round(x, 0), nsmall = 0L, scientific = FALSE)

      res$gg = ggplot(data.frame(px = res$px, py = res$py),
            aes(x = .data[["px"]], y = .data[["py"]])) + xlab(xlab) + ylab(ylab) +
            geom_hex(bins = bins, ...) +
            scale_fill_gradient(name = "count", trans = "log", labels = fmt()) + 
          geom_line(aes(x = .data[["x"]], y = .data[["y"]]),
                    data = data.frame(x = res[[1]], y = res$sd), color = "red")
              
      if (plot) print(res$gg)
      
      return(invisible(res))
})

## ==========================================================================
## meanSdPlot method for ExpressionSet, vsn
## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
setMethod("meanSdPlot", signature="ExpressionSet", definition =
   function(x, ranks=TRUE, xlab = ifelse(ranks, "rank(mean)", "mean"),
            ylab = "sd", pch  = ".", plot = TRUE, ...)
         meanSdPlot(exprs(x), ranks=ranks, xlab=xlab, ylab=ylab, pch=pch, plot=plot, ...))

setMethod("meanSdPlot", signature="vsn", definition =
   function(x, ranks=TRUE, xlab = ifelse(ranks, "rank(mean)", "mean"),
            ylab = "sd", pch  = ".", plot = TRUE, ...)
         meanSdPlot(exprs(x), ranks=ranks, xlab=xlab, ylab=ylab, pch=pch, plot=plot, ...))

setMethod("meanSdPlot", signature="MAList", definition =
   function(x, ranks=TRUE, xlab = ifelse(ranks, "rank(mean)", "mean"),
            ylab = "sd", pch  = ".", plot = TRUE, ...)
         meanSdPlot(with(x, cbind(A+M/2, A-M/2)),
                              ranks=ranks, xlab=xlab, ylab=ylab, pch=pch, plot=plot, ...))

## Test gradient calculation in vsn C code
##     both for the likelihood with given mu_k, sigma and
##     for the profile likelihood.
##

library("vsn")
options(error=recover)

switch(1L,
       { data("kidney")
         x = exprs(kidney)
         nrpt = 25L ## number of points p0 from which to consider
         strata = cut(1:nrow(x), 3) },
       { data("lymphoma")
         x = exprs(lymphoma)[, 1:8]*1.0
         mode(x) = "double"
         nrpt  = 3L ## number of points p0 from which to consider
         strata = factor(rep(1L, nrow(x)))
       })

nrpar = 2L*ncol(x)*nlevels(strata)
k     = ncol(x)*nlevels(strata)
eps   = 1e-6

norm = function(x) sqrt(sum(x*x))

v = new("vsnInput", x=x,
  pstart=array(as.numeric(NA), dim=c(nlevels(strata), ncol(x), 2L)),
  strata=strata, ordered=TRUE)

fit = vsn2(x, lts.quantile=1, strata=strata)


doit = function(nm, fun) {
  cat("Wait for", nrpt, "points: ")
  
  df = array(NA, dim=c(nrpar, nrpt, 3L))
  for (ip in 1:nrpt) {
    cat(ip, "")
    p0 = as.vector(coef(fit))
    p0 = p0 + runif(length(p0), min=-.2, max=.2)
    
    df[,ip,1L] = fun(p0)[-1L]

    for(il in 1:nrpar) {
      dp = eps*(il==1:nrpar)
      fn = fun(cbind(p0-dp, p0+dp))[1L, ]
      grn = diff(fn)/norm(2*dp)
      stopifnot(is.finite(grn))
      df[il,ip,2L]  = grn
      }

    df[,ip,3L] = p0
  }
  cat("\n\n")
  x11(width=14, height=7)
  par(mfrow=c(2, ncol(x)*nlevels(strata)+1))
  lj = list(offset=1:(ncol(x)*nlevels(strata)),
            factor=ncol(x)*nlevels(strata)+(1:(ncol(x)*nlevels(strata))))
  for(j in seq(along=lj)){
    for(i in seq(along=lj[[j]])) {
      il = lj[[j]][i]
      plot(df[il,,1], df[il,,2], pch=16, xlab="analytic", ylab="numeric",
           main=sprintf("%s %s %d", nm, names(lj)[j], i))
      abline(a=0, b=1, col="blue")
    }
    hist(df[lj[[j]],,1]-df[lj[[j]],,2], col="orange", xlab="difference",
       main=paste(names(lj)[j], "s", sep=""), breaks=30)
    abline(v=0, col="blue")
  }
}

graphics.off()

doit("prof", function(p) logLik(v, p))
doit("ref",  function(p) logLik(v, p, mu = rowMeans(x), sigsq = mean(diff(t(x))^2)))

## It would be nice if we could use S4 to dispatch on the class of the second argument
##   "newdata" as well, but unfortunately the generic function "predict" from
##   package "stats" does not define a second argument. We use that function 
##   anyway to hang the following method upon, but will need to do the dispatching
##   manually.

setMethod("predict", signature("vsn"),
  function(object, newdata, strata=object@strata, log2scale=TRUE, useDataInFit=FALSE) {

    stopifnot(validObject(object), validObject(newdata),
              length(log2scale)==1L,    is.logical(log2scale),    !is.na(log2scale),
              length(useDataInFit)==1L, is.logical(useDataInFit), !is.na(useDataInFit))

    ## order matters here:
    classes = c("ExpressionSet", "NChannelSet", "RGList", "AffyBatch", "matrix", "numeric")
    for(cl in classes)
      if(is(newdata, cl))
        return(do.call(paste("predict_vsn", cl, sep="_"),
                       args = list(object=object, newdata=newdata,
                         strata=strata, log2scale=log2scale, useDataInFit)))
               
    stop(sprintf("Expecting the class of 'newdata' to be one of: %s, but found %s.",
                 paste("'", classes, "'", sep="", collapse=", "),
                 class(newdata)))
  })


##-----------------------------------------------------------------
## Here's the meat - all the other methods are just bureaucracy
##-----------------------------------------------------------------
predict_vsn_matrix = function(object, newdata, strata, log2scale, useDataInFit) {

  if(useDataInFit) {
    
    hy = exprs(object)
    
  } else {

    storage.mode(newdata) = "double"
    if(length(strata)==0L) {
      strata = rep(1L, nrow(newdata))
    } else {
      if(length(strata)!=nrow(newdata))
        stop("'nrow(newdata)' must match 'strata'.")
      strata = as.integer(int2factor(strata))
    }

    hy = vsn2trsf(x = newdata,
                  p = object@coefficients,
                  strata = strata,
                  hoffset = if(log2scale) object@hoffset else NULL,
                  calib = object@calib)
    attributes(hy) = attributes(newdata)
      
  }
  
  hy
    
}

##-----------------------------------------------------------------
predict_vsn_numeric = function(object, newdata, ...) {
  dim(newdata) = c(length(newdata), 1)
  predict_vsn_matrix(object, newdata=newdata, ...)
}

##-------------------------------------------------------------------------
predict_vsn_AffyBatch = predict_vsn_ExpressionSet = function(object, newdata, ...) {
  exprs(newdata) = predict_vsn_matrix(object, newdata=exprs(newdata), ...)
  return(newdata)
} 

##-------------------------------------------------------------------------
predict_vsn_NChannelSet = function(object, newdata, strata, log2scale, useDataInFit,
  backgroundsubtract=FALSE, foreground=c("R","G"), background=c("Rb", "Gb")) {

  if(!useDataInFit) {
    x = MatrixFromNChannelSet(newdata, backgroundsubtract=backgroundsubtract,
                              foreground=foreground, background=background)
  } else {
    x = NULL
  }

  mat = predict_vsn_matrix(object, newdata=x, strata=strata,
    log2scale=log2scale, useDataInFit=useDataInFit)

  d    = ncol(newdata)
  nrch = ncol(mat)%/%d
  stopifnot(ncol(mat)%%d==0L)
  chn = attr(object, "ChannelNames")
  stopifnot(is.character(chn), length(chn)==nrch)

  args = vector(mode="list", length=nrch+1L)
  args[[1L]] = "lockedEnvironment"
  for(i in seq_len(nrch))
      args[[i+1L]] = mat[, seq_len(d)+(i-1L)*d, drop=FALSE]
  names(args) = c("storage.mode", chn)
    
  assayData(newdata) = do.call(assayDataNew, args)
  if(!(is(newdata, "NChannelSet") && validObject(newdata)))
    stop("Failed to create a valid 'NChannelSet'.")
  return(newdata)
}

  
##-------------------------------------------------------------------------
predict_vsn_RGList = function(object, newdata, ...)
  predict_vsn_NChannelSet(object, as(newdata, "NChannelSet"), ...)
  

##------------------------------------------------------------
## a wrapper for vsn to be used as a normalization method in
## the package affy
## D P Kreil <bioc07@kreil.org>
##------------------------------------------------------------
normalize.AffyBatch.vsn=function(abatch,
                                 reference,
                                 strata = NULL,
                                 subsample = if (nrow(exprs(abatch))>30000L) 30000L else 0L, 
                                 subset,
                                 log2scale = TRUE,
                                 log2asymp = FALSE,
                                 ...) {

  if(is.na(log2scale)||is.na(log2asymp)||(log2scale&&log2asymp))
    stop("'log2asymp' and 'log2scale' must not both be TRUE, and not be NA.")
  
  ind = indexProbes(abatch,"pm")
  if(!missing(subset))
    ind = ind[subset]
  ind = unlist(ind)
       
  vsn2res = vsn2(intensity(abatch)[ind,],reference=reference,
                 returnData=FALSE,subsample=subsample,...)
  
  description(abatch)@preprocessing = c(description(abatch)@preprocessing, 
                       list(vsnReference=vsn2res))
  
  trsfx = predict(vsn2res,newdata=intensity(abatch),log2scale=log2scale)

  ## irrelevant affine transformation for ~ log2(x) for x>>1
  if (log2asymp)
    trsfx=(trsfx-log(2))/log(2)
   
  intensity(abatch)<-2^trsfx
  return(abatch)
}

plotVsnLogLik = function(object,
              p,
              whichp = 1:2,
              expand = 1,
              ngrid = 31L,
              fun = logLik,
              main = "log likelihood",
              ...) {

  stopifnot(length(whichp)==2L, ngrid>=2L)

  if(length(expand)==1)
    expand=rep(expand,2)

  d = ncol(object)
  stopifnot(2L*d*nlevels(object@strata)==length(p))

  psteps = sapply(1:2, function(k) {
    z = ((whichp[k]-1) %/% nlevels(object@strata))
    i = (z %% d) + 1L
    aorb = z %/% d
    stopifnot(aorb %in% c(0L,1L))
    if(aorb==0) {
      p[whichp[k]] + seq(-expand[1L], +expand[1L], length=ngrid)
    } else {
      p[whichp[k]] + seq(-expand[2L], +expand[2L], length=ngrid)
    }
  })

  pgrid = expand.grid(psteps[,1], psteps[,2], KEEP.OUT.ATTRS = FALSE)
  psamp = matrix(p, nrow=length(p), ncol=nrow(pgrid))
  for(i in 1:2)
    psamp[whichp[i], ] = pgrid[, i]

  ll = fun(object, psamp, ...)
  pgrid$logLik = ll[1, ]

  print(levelplot(logLik ~ Var1*Var2, data=pgrid, main=main,
       col.regions=colorRampPalette(c("#FFFFCC","#FFEDA0","#FED976","#FEB24C","#FD8D3C","#FC4E2A","#E31A1C","#BD0026","#800026"))(128)))
  
  return(invisible(pgrid))
}    

## I copied this file from https://hedgehog.fhcrc.org/bioconductor/trunk/madman/Rpacks/convert/R/RGList_to_NChannelSet.R
## on 15.2.2012. The subversion long stated for the file of origin:
##     "Last Changed Rev: 36027, Last Changed Date: 2008-12-16 22:48:47 +0100 (Tue, 16 Dec 2008)"


setAs("RGList", "NChannelSet", function(from) {

  ## assayData
  assayData <- with(from, {
    if (!exists("other", inherits=FALSE)) {
      elts <- list(R=R, G=G)
    } else {
      if (is.null(names(other)) ||
          !all(sapply(names(other), nzchar)))
        stop(paste("RGList 'other' elements must be named, found '",
                   paste(names(other), collapse="', '"),
                   "'", sep=""))
      bad <- names(other) %in% c("R", "G", "Rb", "Gb")
      if (any(bad))
        stop(paste("RGList 'other' elements contain reserved names '",
                   paste(names(other)[bad],
                         collapse="', '"),
                   "'", sep=""))
      
      elts <- c(R=R, G=G, other)
    }
    if (exists("Rb", inherits=FALSE))
      elts[["Rb"]] <- Rb
    if (exists("Gb", inherits=FALSE))
      elts[["Gb"]] <- Gb
    do.call(assayDataNew,
            c(storage.mode="lockedEnvironment", elts))
  })
  
  ## phenoData
  phenoData <- 
    if (!is.null(from$targets)) {
      new("AnnotatedDataFrame", data=from$targets)
    } else {
      new("AnnotatedDataFrame",
          data=data.frame(rep(0, ncol(from)))[,FALSE])
    }
  ## featureData
  fData <-
    if (!is.null(from$genes))
      from$genes
    else
      data.frame(x=rep(0,nrow(from)))[,FALSE]
  if (!is.null(rownames(assayData[["R"]])))
    row.names(fData) <- rownames(assayData[["R"]])
  featureData <- new("AnnotatedDataFrame", data = fData)
  if (!is.null(from$weights))
    if ("weights" %in% names(df))
      warning("RGList 'genes' contains column 'weights'; 'wt.fun' weights discarded")
    else
      featureData[["weights",
                   labelDescription="calculated, from RGList"]] <-
                     from$weights
  ## experimentData
  other <- 
    if (!is.null(from$source))
      list(source=from$source)
    else
      list()
  experimentData <- new("MIAME",
                        other=c("converted from marrayRaw", other))
  new("NChannelSet",
      assayData = assayData,
      featureData = featureData,
      phenoData = phenoData,
      experimentData=experimentData)
})

setMethod("[", "vsnInput",
  function(x, i, j, ..., drop=FALSE) {
    stopifnot(!drop, length(list(...))==0)

    ## rows
    if(!missing(i)) {
      x@x = x@x[i,,drop=FALSE]
 
      if(nrow(x@reference)>0)
        x@reference = x@reference[i,,drop=FALSE]
    
      if(length(x@strata)>0)
        x@strata = x@strata[i,drop=FALSE]
    }

    ## columns
    if(!missing(j)) {
      x@x = x@x[,j,drop=FALSE]
      x@pstart = x@pstart[,j,,drop=FALSE]
    }
    
    return(x)
  })
       
setMethod("nrow", signature("vsnInput"), function(x) nrow(x@x))
setMethod("ncol", signature("vsnInput"), function(x) ncol(x@x))
setMethod("dim",  signature("vsnInput"), function(x) dim(x@x))

setMethod("show", signature("vsnInput"),
  function(object) {
    cat(class(object), sprintf("object for n=%d features and d=%d samples.\n",
      nrow(object), ncol(object)))
    if(length(object@strata)>0)
      cat(sprintf("strata: %d levels\n", nlevels(object@strata)))
    if(nrow(object@reference)==0)
      cat("No prior reference fit available.\n")
    else
      cat(sprintf("Has reference fit parameters; sigsq=%g\n", round(object@reference@sigsq, 3)))
  })


setMethod("logLik", signature(object="vsnInput"), vsnLogLik)

test_that("Make sure that if the calib='affine' option is used in 'vsn2', the 'predict' method works on the resulting 'vsn' object.", {
  ## motivated by https://support.bioconductor.org/p/92235/
  set.seed(0xbadbeef)  
  nr   <- 512
  nc   <- 8
  dat  <- exp(matrix(rnorm(nr*nc), nr, nc) + log(seq_len(nr)))
  fit1 <- vsn2(dat)
  fit2 <- vsn2(dat, calib = "none")
  vs1  <- predict(fit1, newdata = dat)
  vs2  <- predict(fit2, newdata = dat)
})

---
title: "Verifying and assessing the performance with simulated data"
package: vsn
output: 
  BiocStyle::html_document:
    toc: true
vignette: >
  %\VignetteIndexEntry{Verifying and assessing the performance with simulated data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

# Verifying and assessing the performance with simulated data

Please see the .Rnw and .pdf file in the inst/vignettes folder of the vsn package.
It takes several hours to run, therefore it is not included here.

---
title: "Introduction to vsn"
author: "Wolfgang Huber"
package: vsn
output: 
  BiocStyle::html_document:
    toc: true
bibliography: vsn.bib
vignette: >
  %\VignetteIndexEntry{Introduction to vsn (HTML version)}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding[utf8]{inputenc}
---

```{r setup, echo = FALSE, results = "hide"}
options(signif = 3, digits = 3)
knitr::opts_chunk$set(tidy = FALSE, cache = TRUE, autodep = TRUE, fig.height = 5.5,
                      message = FALSE, error = FALSE, warning = TRUE)
set.seed(0xdada)
```

# Getting started{#sec:overv}

VSN is a method to preprocess microarray intensity data.  This can be as simple as

```{r overv1, results = "hide"}
library("vsn")
data("kidney")
xnorm = justvsn(kidney)
```

where `kidney` is an `ExpressionSet` object with unnormalised data and `xnorm` the resulting `ExpressionSet` with calibrated and glog$_2$-transformed data.

```{r overv2}
M = exprs(xnorm)[,1] - exprs(xnorm)[,2]
```

produces the vector of generalised log-ratios between the 
data in the first and second column.

VSN is a model-based method, and the more explicit way of 
doing the above is 

```{r overv3, results = "hide"}
fit = vsn2(kidney)
ynorm = predict(fit, kidney)
```
```{r overv4, echo = FALSE, results = "hide"}
stopifnot(
  max(abs(exprs(xnorm) - exprs(ynorm))) < 1e-16, 
  max(abs(exprs(xnorm) - exprs(fit)))   < 1e-16,
  identical(dim(exprs(xnorm)), dim(exprs(ynorm))), 
  identical(dim(exprs(xnorm)), dim(exprs(fit))))
``` 

where `fit` is an object of class `vsn` that contains
the fitted calibration and transformation parameters, and the method
`predict` applies the fit to the data.
The two-step protocol is useful when you want to fit the parameters on a
subset of the data, e.g. a set of control or spike-in features,
and then apply the model to the complete set of data^[See Section \@ref(sec:spike) for details.].
Furthermore, it enables further inspection of the `fit` object, e.g., for the
purpose of quality assessment.

Besides `ExpressionSet`s, there are also `justvsn` 
methods for `AffyBatch` objects from the `r Biocpkg("affy")` package and
`RGList` objects from the `r Biocpkg("limma")` package. They are described 
in this vignette.

The so-called glog$_2$ (short for generalised logarithm) 
is a function that is like the logarithm (base 2) for large values 
(large compared to the amplitude of the background noise),
but is less steep for smaller values. While the $\log_2$ function diverges to
negative infinity at zero, the glog$_2$ function approximates a straight line of
finite slope around 0^[See Section \@ref(sec:shrink) for a visualization.].
Differences between the transformed values are the generalised log-ratios. These 
are shrinkage estimators of the logarithm of the fold change.
The usual log-ratio is another example for an estimator^[In statistics,
the term estimator is used to denote any algorithm
that calculates a value from measured data. 
This value is intended to correspond to the true value of a parameter of the 
underlying process that generated the data. Depending on the amount of the 
available data and the quality of the estimator, the intention 
may be more or less satisfied.] of log fold change. 
There is also a close relationship between background correction of 
the intensities and the variance properties of the different estimators. Please
see Section \@ref(sec:shrink) for more explanation of these issues. 

How does VSN work? There are two components: First, an affine
transformation whose aim is to calibrate systematic experimental
factors such as labelling efficiency or detector sensitivity.  Second,
a glog$_2$ transformation whose aim is variance stabilisation.

An affine transformation is simply a shifting and scaling of
the data, i.e. a mapping of the form $x\mapsto (x-a)/s$ with offset
$a$ and scaling factor $s$.  By default, a different offset and a
different scaling factor are used for each column, but the same for
all rows within a column. There are two parameters of the function
`vsn2` to control this behaviour: With the parameter
`strata`, you can ask `vsn2` to choose different
offset and scaling factors for different groups ("strata") of
rows. These strata could, for example, correspond to sectors on the
array^[See Section \@ref(sec:strata).]. With the parameter
`calib`, you can ask `vsn2` to choose the same
offset and scaling factor throughout^[See
Section \@ref(sec:nocalib).]. This can be useful, for example, if the
calibration has already been done by other means, e.g., quantile
normalisation.

## Limitations

### Other determinants of variance

The VSN transformation only addresses the
dependence of the variance on the mean intensity. There may be important other
factors influencing the variance, including gene-inherent properties,
sample-inherent properties such as different tightness of transcriptional control in different
conditions, or technical factors. These need to be addressed by other, more specialized models.

### Numerical stability and convergence

VSN does not work for all possible inputs. If you encounter an error message like
"L-BFGS-B needs finite values of 'fn'"^[For technical readers: fitting the VSN model
involves optimisation of a multivariate, non-quadratic likelihood. For bad or unsuitable data,
this function may be non-convex, and in particular, it may be flat in some directions
of parameter space. In such cases, the optimiser can run out into infinity, which
results in this error message. One could contemplate a version of `vsn2` that avoids this
by using regularisation. In any case, the current behaviour alerts users to 
problems rather than simply glossing over.], please check whether one of these 
problems is present:

- A bug in the code that does the preprocessing of the data that go in.
- A problem with data quality or experimental design, e.g., incompatible measurements assembled together into one input matrix.
- The noise or variance characteristics of the data are just not suitable for the VSN model.

# Running VSN on data from a single two-colour array

The dataset `kidney` contains example data from a spotted cDNA
two-colour microarray on which cDNA from two adjacent tissue samples
of the same kidney were hybridised, one labeled in green (Cy3), one in
red (Cy5).  The two columns of the matrix `exprs(kidney)`
contain the green and red intensities, respectively. A local
background estimate^[See Section \@ref(sec:shrink) for more
on the relationship between background correction and variance
stabilising transformations.] was calculated by the image analysis
software and subtracted, hence some of the intensities in
`kidney` are close to zero or negative. In
Figure \@ref(fig:nkid-scp) you can see the scatterplot of the
calibrated and transformed data. For comparison, the scatterplot of
the log-transformed raw intensities is also shown. The code below involves
some data shuffling to move the data into datasframes for `ggplot`.

```{r nkid-scp, results = "hide", fig.cap = paste0("Scatterplots of the kidney example data, which were obtained from a two-color cDNA array by quantitating spots and subtracting a local background estimate. a) unnormalised and $\\log_2$-transformed. b) normalised and transformed with VSN. Panel b shows the data from the complete set of ", nrow(kidney), " spots on the array, Panel a only the ", sum(0==rowSums(exprs(kidney)<=0)), " spots for which both red and green net intensities were greater than 0. Those spots which are missing in Panel a are coloured in orange in Panel b."), fig.height = 4, fig.wide = TRUE}
library("ggplot2")
allpositive = (rowSums(exprs(kidney) <= 0) == 0)

df1 = data.frame(log2(exprs(kidney)[allpositive, ]),
                 type = "raw",
		 allpositive = TRUE)
df2 = data.frame(exprs(xnorm),
                 type = "vsn",
		 allpositive = allpositive)
df = rbind(df1, df2)
names(df)[1:2] = c("x", "y") 

ggplot(df, aes(x, y, col = allpositive)) + geom_hex(bins = 40) +
  coord_fixed() + facet_grid( ~ type)
```

To verify the variance stabilisation, there is the function `meanSdPlot`. For each
feature $k=1,\ldots,n$ it shows the empirical standard deviation
$\hat{\sigma}_k$ on the $y$-axis versus the rank of the average
$\hat{\mu}_k$ on the $x$-axis.

\begin{equation}
\hat{\mu}_k     =\frac{1}{d}  \sum_{i=1}^d h_{ki}\quad\quad
\hat{\sigma}_k^2=\frac{1}{d-1}\sum_{i=1}^d (h_{ki}-\hat{\mu}_k)^2
\end{equation}

```{r nkid-sdmean1, fig.cap = "Standard deviation versus rank of the mean", fig.height = 4, fig.small = TRUE}
meanSdPlot(xnorm, ranks = TRUE)
```
```{r nkid-sdmean2, fig.cap = "Standard deviation versus mean", fig.height = 4, fig.small = TRUE}
meanSdPlot(xnorm, ranks = FALSE) 
```

The red dots, connected by lines, show the running median of the standard
deviation^[The parameters used were: window width 10%, window
midpoints 5%, 10%, 15%, ... . It should be said that the proper
way to do is with quantile regression such as provided by the
`r CRANpkg("quantreg")` package---what is done here for these plots is
simple, cheap and should usually be good enough due to the abundance
of data.]. The aim of these plots is to see whether there is a
systematic trend in the standard deviation of the data as a function
of overall expression. The assumption that underlies the usefulness of
these plots is that most genes are not differentially expressed, so
that the running median is a reasonable estimator of the standard
deviation of feature level data conditional on the mean.  After
variance stabilisation, this should be approximately a horizontal
line.  It may have some random fluctuations, but should not show an
overall trend.  If this is not the case, that usually indicates a data
quality problem, or is a consequence of inadequate prior data
preprocessing.  The rank ordering distributes the data evenly along
the $x$-axis. A plot in which the $x$-axis shows the average
intensities themselves is obtained by calling the `plot`
command with the argument `ranks=FALSE`; but this is
less effective in assessing variance and hence is not the default.

The histogram of the generalized log-ratios:

```{r nkid-histM, fig.cap = "Histogram of generalised log-ratios for the kidney example data.", fig.small = TRUE}
hist(M, breaks = 100, col = "#d95f0e")
```

# Running VSN on data from multiple arrays ("single colour normalisation"){#sec:lymphoma} 

The package includes example data from a series of 8 spotted cDNA arrays 
on which cDNA samples from different lymphoma were hybridised together 
with a reference cDNA [@Alizadeh].

```{r lymphoma}
data("lymphoma")
dim(lymphoma)
```

The 16 columns of the `lymphoma` object contain the red and
green intensities, respectively, from the 8 slides.

```{r pDatalym}
pData(lymphoma)
``` 

We can call `justvsn` on all of them at once:

```{r lymjustvsn, results = "hide"}
lym = justvsn(lymphoma)
``` 
```{r lym-sdmean, fig.cap = "Standard deviation versus rank of the mean for the lymphoma example data", fig.height = 4, fig.small = TRUE}
meanSdPlot(lym)
```

We see that the variance stabilisation worked. As above, we can obtain the generalised 
log-ratios for each slide by subtracting the common reference intensities from those for the 8 samples:

```{r lym-M, fig.cap = "MA-plot for slide DLCL-0032 from the lymphoma example data. A false-colour representation of the data point density is used, in addition the 100 data points in the least dense regions are plotted as dots.", fig.small = TRUE}
iref = seq(1, 15, by=2)
ismp = seq(2, 16, by=2)
M = exprs(lym)[,ismp]-exprs(lym)[,iref] 
A =(exprs(lym)[,ismp]+exprs(lym)[,iref])/2
colnames(M) = lymphoma$sample[ismp]
colnames(A) = colnames(M)

j = "DLCL-0032"
smoothScatter(A[,j], M[,j], main=j, xlab="A", ylab="M", pch=".")
abline(h=0, col="red")
```


# Running VSN on Affymetrix genechip data{#sec:affy}

The package `r Biocpkg("affy")` provides excellent functionality for
reading and processing Affymetrix genechip data, and you are encouraged
to refer to the documentation of the package `r Biocpkg("affy")` 
for more information about data structures and methodology. 
%
The preprocessing of Affymetrix genechip data involves
the following steps:
(i) background correction,
(ii) between-array normalization,
(iii) transformation and (iv) summarisation.
The VSN method addresses steps (i)--(iii).
For the summarisation, I recommend to use the RMA method [@RMA],
and a simple wrapper that provides all of these is provided through 
the method `vsnrma`.

```{r loadaffy1, results = "hide", warning = FALSE}
library("affydata")
data("Dilution")
d_vsn = vsnrma(Dilution)
``` 
For comparison, we also run `rma`.
```{r loadaffy2, results = "hide"}
d_rma = rma(Dilution)
``` 

```{r affy, fig.height=3.75, fig.wide = TRUE, fig.cap = paste0("Results of `vsnrma` and `rma` on the Dilution data. Array 1 was hybridised with ", pData(Dilution)$liver[1], "$\\mu$g RNA from liver, Array 3 with ", pData(Dilution)$liver[3], "$\\mu$g of the same RNA.")}
par(mfrow = c(1, 3))
ax = c(2, 16)

plot(exprs(d_vsn)[,c(1,3)], main = "vsn: array 1 vs 3", asp = 1, xlim = ax, ylim = ax, pch = ".")

plot(exprs(d_rma)[,c(1,3)], main = "rma: array 1 vs 3", asp = 1, xlim = ax, ylim = ax, pch = ".")

plot(exprs(d_rma)[,1], exprs(d_vsn)[,1], 
     xlab = "rma", ylab = "vsn", asp = 1, xlim = ax, ylim = ax, main = "array 1", pch = ".")
abline(a = 0,  b =1, col = "#ff0000d0")
```

Both methods control the variance at low intensities, but we see that
VSN does so more strongly. See also Section \@ref(sec:shrink) for further 
discussion on the VSN shrinkage.

# Running VSN on RGList objects{#sec:limma}

There is a `justvsn` method for `RGList` objects.
Usually, you will produce an `RGList` from your own data using the
`read.maimages` from the `r Biocpkg("limma")` package. Here,
for the sake of demonstration, we construct an `RGList` from
`lymphoma`.

```{r loadlimma, results = "hide"} 
library("limma")
wg = which(lymphoma$dye=="Cy3")
wr = which(lymphoma$dye=="Cy5")

lymRG = new("RGList", list(
  R=exprs(lymphoma)[, wr],
  G=exprs(lymphoma)[, wg]))

lymNCS = justvsn(lymRG)
``` 

The `justvsn` method for `RGList` 
converts its argument into an `NChannelSet`, using a copy 
of the coercion method from Martin Morgan in the package `r Biocpkg("convert")`.
It then passes this on to 
the `justvsn` method for `NChannelSet`. 
The return value is an `NChannelSet`, as shown below.

```{r lymNCS}
lymNCS
``` 

Note that, due to the flexibility in the amount and 
quality of metadata that is in an `RGList`, and due to 
differences in the implementation of these classes,
the transfer of the metadata into the `NChannelSet` 
may not always produce the expected results, and that some
checking and often further dataset-specific postprocessing
of the sample metadata and the array feature annotation
is needed. For the current example, we 
construct the `AnnotatedDataFrame` object `adf` 
and assign it into the `phenoData` slot of `lymNCS`.

```{r addmeta}
vmd = data.frame(
  labelDescription = I(c("array ID", "sample in G", "sample in R")),
  channel = c("_ALL", "G", "R"),
  row.names = c("arrayID", "sampG", "sampR"))

arrayID = lymphoma$name[wr]
stopifnot(identical(arrayID, lymphoma$name[wg])) 

## remove sample number suffix 
sampleType = factor(sub("-.*", "", lymphoma$sample)) 

v = data.frame(
  arrayID = arrayID,
  sampG   = sampleType[wg],
  sampR   = sampleType[wr])
v
              
adf = new("AnnotatedDataFrame", 
  data = v,
  varMetadata = vmd)

phenoData(lymNCS) = adf 
``` 

Now let us combine the red and green values from each array
into the glog-ratio M
and use the linear modeling tools from `r Biocpkg("limma")` to find
differentially expressed genes (note that it is often suboptimal
to only consider M, and that taking into account absolute
intensities as well can improve analyses).

```{r lymM}
lymM = (assayData(lymNCS)$R - 
        assayData(lymNCS)$G)
```

```{r design}
design = model.matrix( ~ lymNCS$sampR)
lf = lmFit(lymM, design[, 2, drop=FALSE])
lf = eBayes(lf)
``` 

The following plots show the resulting $p$-values and
the expression profiles of the genes corresponding to the 
top 5 features.

```{r limma, fig.wide = TRUE, fig.cap = paste("Left: histogram of p-values from the moderated $t$-test between the",  levels(lymNCS$sampR)[1], "and",  levels(lymNCS$sampR)[2], "groups on the `lymM` values. Right: M-values for the 5 genes with the smallest p-values.")}
par(mfrow=c(1,2))
hist(lf$p.value, breaks = 100, col="orange") 
pdat = t(lymM[order(lf$p.value)[1:5],])
matplot(pdat, lty = 1, type = "b", lwd = 2, col=hsv(seq(0,1,length=5), 0.7, 0.8), 
  ylab = "M", xlab = "arrays")
``` 


## Background subtraction

Many image analysis programmes for microarrays provide local background
estimates, which are typically calculated from the fluorescence signal
outside, but next to the features. These are not always useful. Just
as with any measurement, these local background estimates are also subject
to random measurement error, and subtracting them from the foreground
intensities will lead to increased random noise in the signal. On the
other hand side, doing so may remove systematic artifactual drifts in the
data, for example, a spatial gradient. 

So what is the optimal analysis strategy, should you subtract local
background estimates or not? The answer depends on the properties of
your particular data. VSN itself estimates and subtracts an
over-all background estimate (per array and colour, see
Section \@ref(sec:calib)), so an additional local background correction
is only useful if there actually is local variability across an array,
for example, a spatial gradient.

Supposing that you have decided to subtract the local background
estimates, how is it done? 
When called with the argument `backgroundsubtract=TRUE`^[
Note that the default value for this parameter is `FALSE`.], 
the `justvsn` method will subtract local background estimates in
the `Rb` and `Gb` slots of the incoming `RGList`.
To demonstrate this, we construct an `RGList` object `lymRGwbg`.

```{r makebg}
rndbg = function(x, off, fac)
   array(off + fac * runif(prod(dim(x))), dim = dim(x))

lymRGwbg    = lymRG
lymRGwbg$Rb = rndbg(lymRG, 100, 30)
lymRGwbg$Gb = rndbg(lymRG,  50, 20)
``` 

In practice, of course, these values will be read from the image
quantitation file with a function such as `read.maimages`
that produces the `RGList` object. We can call `justvsn`

```{r justvsnwbg, message = FALSE}
lymESwbg = justvsn(lymRGwbg[, 1:3], backgroundsubtract=TRUE)
``` 

Here we only do this for the first 3 arrays to save compute time.


## Print-tip groups{#sec:strata}

By default, VSN computes one normalisation transformation with a common set of parameters
for all features of an array (separately for each colour if it is a multi-colour
microarray), see Section \@ref(sec:calib).  Sometimes, there is a need for stratification
by further variables of the array manufacturing process, for example, print-tip groups
(sectors) or microtitre plates. This can be done with the `strata` parameter of
`vsn2`.

The example data that comes with the package does not directly provide the information
which print-tip each feature was spotted with, but we can easily reconstruct it:

```{r pinId}
ngr = ngc = 4L
nsr = nsc = 24L
arrayGeometry = data.frame(
  spotcol = rep(1:nsc, times = nsr*ngr*ngc),
  spotrow = rep(1:nsr, each = nsc, times=ngr*ngc),
  pin = rep(1:(ngr*ngc), each = nsr*nsc))
```

and call

```{r strata, results = "hide"}
EconStr = justvsn(lymRG[, 1], strata = arrayGeometry$pin)
```

To save CPU time, we only call this on the first array. We compare the 
result to calling `justvsn` without `strata`,

```{r nostrata, results = "hide"}
EsenzaStr = justvsn(lymRG[, 1])
```

A scatterplot comparing the transformed red intensities,
using the two models, is shown in Figure \@ref(fig:figstrata2).

```{r figstrata2, fig.small = TRUE, fig.cap = "Scatterplot of normalised and transformed intensities for the red channel of Array 1. Values on the $x$-axis correspond to normalisation without strata (`EsenzaStr`), values on the $y$-axis to normalisation with strata (`EconStr`). The different colours correspond to the 16 different strata."}
j = 1
plot(assayData(EsenzaStr)$R[,j],
     assayData(EconStr)$R[,j],
     pch = ".", asp = 1, 
     col = hsv(seq(0, 1, length=ngr*ngc), 
       0.8, 0.6)[arrayGeometry$pin], 
     xlab = "without strata", 
     ylab = "print-tip strata",
     main = sampleNames(lymNCS)$R[j])
```

# Missing values{#sec:miss}

The parameter estimation algorithm of VSN is able to deal with
missing values in the input data. To demonstrate this, we generate an 
`ExpressionSet` `lym2` in which about 10% of all intensities 
are randomly missing,

```{r miss1}
lym2 = lymphoma
nfeat = prod(dim(lym2))
wh = sample(nfeat, nfeat/10)
exprs(lym2)[wh] = NA
table(is.na(exprs(lym2)))
``` 

and call `vsn2` on it.

```{r miss2, results = "hide"}
fit1 = vsn2(lymphoma, lts.quantile=1)
fit2 = vsn2(lym2, lts.quantile=1)
``` 

The resulting fitted parameters are not identical, but very similar, 
see Figure \@ref(fig:miss).
%
```{r miss, fig.wide = TRUE, fig.cap="Scatterplots of fitted parameters, values on the $x$-axis correspond to normalisation without missing data (`fit1`), values on the $y$-axis to normalisation with around 10% missing data (`fit2`)."}
par(mfrow=c(1,2))
for(j in 1:2){
  p1 = coef(fit1)[,,j]
  p2 = coef(fit2)[,,j]
  d  = max(abs(p1-p2))
  stopifnot(d < c(0.05, 0.03)[j])
  plot(p1, p2, pch = 16, asp = 1,
    main = paste(letters[j], 
      ": max diff=", signif(d,2), sep = ""),
    xlab = "no missing data",
    ylab = "10% of data missing")
  abline(a = 0, b = 1, col = "blue")
}
``` 

Note that `p1` and `p2` would differ more if we used a different value than 1 for the `lts.quantile` argument in the above calls of `vsn2`.  This is because the outlier removal algorithm will, for this dataset, identify different sets of features as outliers for `fit1` and `fit2` and consequently the optimisation result will be slightly different; this difference is arguably negligible compared to the noise level in the data.


# Normalisation with 'spike-in' probes{#sec:spike}

Normally, VSN uses all features on the array to fit the calibration
and transformation parameters, and the algorithm relies, to a certain
extent, on the assumption that most of the features' target genes are
not differentially expressed (see also Section \@ref(sec:mostunch)). 
If certain features are known to correspond to, or not to
correspond to, differentially expressed targets, then we can help the
algorithm by fitting the calibration and transformation parameters
only to the subset of features for which the "not
differentially expressed" assumption is most appropriate, and then
applying the calibration and transformation to **all**
features. For example, some experimental designs provide "spike-in"
control spots for which we know that their targets' abundance is the
same across all arrays (and/or colours).

For demonstration, let us assume that in the `kidney` data,
features 100 to 200 are spike-in controls. Then we can obtain a
normalised dataset `nkid` as follows.

```{r spikein, warning = FALSE, message = FALSE}
spikeins = 100:200
spfit = vsn2(kidney[spikeins,], lts.quantile=1)
nkid = predict(spfit, newdata=kidney)
``` 

Note that if we are sufficiently confident that the `spikeins`
subset is really not differentially expressed, and also has no
outliers for other, say technical, reasons, then we can set the
robustness parameter `lts.quantile` to 1.  This corresponds no
robustness (least sum of squares regression), but makes most use of
the data, and the resulting estimates will be more precise, which may
be particularly important if the size of the `spikeins` set is
relatively small.

Not that this explicit subsetting strategy is designed for features
for which we have a priori knowledge that their normalised
intensities should be unchanged. There is no need for you to devise
data-driven rules such as using a first call to VSN to get a
preliminary normalisation, identify the least changing features, and
then call VSN again on that subset. This strategy is already built
into the VSN algorithm and is controlled by its `lts.quantile`
parameter. Please see Section \@ref(sec:mostunch) and
reference [@Huber:2003:SAGMB] for details.


# Normalisation against an existing reference dataset{#sec:ref}

So far, we have considered the joint normalisation of a set of arrays
to each other. What happens if, after analysing a set of arrays in
this fashion, we obtain some additonal arrays? Do we re-run the whole
normalisation again for the complete, new and bigger set of arrays?
This may sometimes be impractical.

Suppose we have used a set of training arrays for setting up a
classifier that is able to discriminate different biological states of
the samples based on their mRNA profile. Now we get new test
arrays to which we want to apply the classifier. Clearly, we do not
want to re-run the normalisation for the whole, new and bigger dataset, as this
would change the training data; neither can we normalise only the
test arrays among themselves, without normalising them towards
the reference training dataset. What we need is a normalisation
procedure that normalises the new test arrays towards the existing
reference dataset without changing the latter.

To simulate this situation with the available example data, 
pretend that the Cy5 channels of the 
`lymphoma` dataset can be treated as 8
single-colour arrays, and fit a model to the first 7.

```{r ref1, results = "hide"}
ref = vsn2(lymphoma[, ismp[1:7]])
``` 

Now we call `vsn2` on the 8-th array, with the output
from the previous call as the reference.

```{r ref2, results = "hide"}
f8 = vsn2(lymphoma[, ismp[8]], reference = ref)
```

We can compare this to what we get if we fit the model to all 8 arrays,

```{r ref3, results = "hide"}
fall = vsn2(lymphoma[, ismp])
```
```{r ref4}
coefficients(f8)[,1,]
coefficients(fall)[,8,]
```
 
and compare the resulting values in the scatterplot shown in Figure \@ref(fig:ref): they are very similar. 

```{r ref, fig.small = TRUE, fig.cap = "Scatterplot of normalised intensities after normalisation by reference ($x$-axis, `f8`) and joint normalisation ($y$-axis, `fall`). There is good agreement."}
plot(exprs(f8), exprs(fall)[,8], pch = ".", asp = 1)
abline(a = 0, b = 1, col = "red")
``` 

```{r hiddenchecks, echo = FALSE}
stopifnot(length(ismp)==8L)
maxdiff = max(abs(exprs(f8) - exprs(fall)[,8])) 
if(maxdiff>0.3)
  stop(sprintf("maxdiff is %g", maxdiff))
``` 

More details on this can be found in the vignettes
*Verifying and assessing the performance with simulated data* 
and
*Likelihood Calculations for vsn*
that come with this package.

# The calibration parameters{#sec:calib}

If $y_{ki}$ is the matrix of uncalibrated data, with $k$ indexing the
rows and $i$ the columns, then the calibrated data $y_{ki}'$ is
obtained through scaling by $\lambda_{si}$ and shifting by $\alpha_{si}$:

\begin{equation}
y_{ki}' = \lambda_{si}y_{ki}+\alpha_{si}
(\#eq:cal)
\end{equation}

where $s\equiv s(k)$ is the so-called _stratum_ for feature $k$. In the
simplest case, there is only one stratum, i.e. the index $s$ is always
equal to 1, or may be omitted altogether. This amounts to assuming that 
the data of all features on an array were subject to the same 
systematic effects, such that an array-wide calibration is sufficient.

A model with multiple strata per array may be useful for spotted
arrays. For these, stratification may be according to
print-tip [@Dudoit578] or PCR-plate [@Huber:2003:HSG]. For
oligonucleotide arrays, it may be useful to stratify the features by
physico-chemical properties, e.g., to assume that features of
different sequence composition attract systematically different levels
of unspecific background signal.

The transformation to a scale where the variance of the data is 
approximately independent of the mean is

\begin{eqnarray}
h_{ki} &=& \text{arsinh}(\lambda_0y_{ki}'+\alpha_0) (\#eq:vs) \\
&=& \log\left(
\lambda_0y_{ki}'+\alpha_0+
\sqrt{\left(\lambda_0y_{ki}'+\alpha_0\right)^2+1}\right),\nonumber
\end{eqnarray}

with two parameters $\lambda_0$ and $\alpha_0$.
Equations \@ref(eq:cal) and \@ref(eq:vs) can be combined, 
so that the whole transformation is given by

\begin{equation}
h_{ki} = \text{arsinh}\left(e^{b_{si}}\cdot y_{ki}+a_{si}\right).
(\#eq:transf)
\end{equation}

Here, 
$a_{si}=\alpha_{si}+\lambda_0\alpha_{si}$ and 
$b_{si}=\log(\lambda_0\lambda_{si})$ 
are the combined calibation and transformation parameters for features from
stratum $s$ and sample $i$. Using the parameter $b_{si}$ as defined 
here rather than $e^{b_{si}}$ appears to make the numerical
optimisation more reliable (less ill-conditioned).

We can access the calibration and transformation parameters through
```{r nkid-calib1}
coef(fit)[1,,]
```

For a dataset with $d$ samples and $s$ strata,
`coef(fit)` is a numeric array with dimensions $(s, d, 2)$. 
For the example data that was used in Section \@ref(sec:overv) to generate 
`fit`, $d=2$ and $s=1$.
`coef(fit)[s, i, 1]`, the first line in the results of the above code chunk, 
is what was called $a_{si}$ in Eqn. \@ref(eq:transf), and
`coef(fit)[s, i, 2]`, the second line, is $b_{si}$. 

## The calibration parameters and the additive-multiplicative error model 

VSN is based on the additive-multiplicative error 
model [@RockeDurbin2001,@Huber:2004:Encyc], which predicts a quadratic variance-mean
relationship of the form [@Huber:2002:Bioinformatics]

\begin{equation}
v(u)=(c_1u+c_2)^2+c_3.
\end{equation}

This is a general parameterization of a parabola with three parameters
$c_1$, $c_2$ and $c_3$. Here, $u$ is the expectation value (mean) of
the signal, and $v$ the variance. $c_1$ is also called the coefficient
of variation, since for large $u$, $\sqrt{v}/u\approx c_1$. The
minimum of $v$ is $c_3$, this is the variance of the additive noise
component. It is attained at $u=-c_2/c_3$, and this is the expectation
value of the additive noise component, which ideally were zero ($c_2=0$), 
but in many applications is different from zero.
Only the behaviour of $v(u)$ for $u\ge -c_2/c_3$ is typically relevant.

The parameters $a$ and $b$ from Equation \@ref(eq:transf)^[I
drop the indices $s$, $k$ and $i$, since for the purpose of this section,
they are passive] and the parameters of the additive-multiplicative
error model are related by [@Huber:2002:Bioinformatics]

\begin{eqnarray}
a&=&\frac{c_2}{\sqrt{c_3}}\nonumber\\ 
e^b&=&\frac{c_1}{\sqrt{c_3}}(\#eq:errmod2trsf)
\end{eqnarray}

This relationship is not 1:1, and it has a divergence at $c_3\to0$; both of these
observations have practical consequences, as explained in the following.

1. The fact that Equations \@ref(eq:errmod2trsf) do not constitute a 1:1
relationship means that multiple parameter sets
of the additive-multiplicative error model can lead to the same
transformation. This can be resolved, for example, if the coefficient of variation $c_1$ is
obtained by some other means than the `vsn2` function. For example, it can be
estimated from the standard deviation of the VSN-transformed data, 
which is, in the approximation of the delta method, 
the same as the coefficient of variation [@Huber:2002:Bioinformatics,@Huber:2003:SAGMB].
Then,

\begin{eqnarray}
c_3&=& c_1^2\, e^{-2b}\nonumber\\ 
c_2&=& c_1  \, a e^{-b}.(\#eq:trsf2errmod)
\end{eqnarray}

2. The divergence for $c_3\to0$ can be a more serious problem. In some
datasets, $c_3$ is in fact very small. This is the case if the size of
the additive noise is negligible compared to the multiplicative noise
throughout the dynamic range of the data, even for the smallest
intensities. In other words, the additive-multiplicative error model
is overparameterized, and a simpler multiplicative-only model would be
good enough. VSN is designed to still produce reasonable results 
in these cases, in the sense that the transformation stabilizes the 
variance (it turns essentially into the usual logarithm transformation),
but the resulting fit coefficients can be unstable. 

The assessment of the precision of the estimated values of $a$ and $b$
(e.g. by resampling, or by using replicate data) is therefore usually
not very relevant; what is relevant is an assessment of the
precision of the estimated transformation, i.e. how much do the
transformed values vary [@Huber:2003:SAGMB].


## More on calibration 

Now suppose the kidney example data were not that well measured, and 
the red channel had a baseline that was shifted by 500 and a scale 
that differed by a factor of $0.25$:

```{r nkid-calib2}
bkid = kidney
exprs(bkid)[,1] = 0.25*(500+exprs(bkid)[,1])
```

We can again call `vsn2` on these data

```{r nkid-calib3, results = "hide"}
bfit = vsn2(bkid)
```
 
```{r nkid-calib4, warning = FALSE, fig.cap = "Scatterplot for the badly biased `bkid` data: log-log scale.", fig.small = TRUE}
plot(exprs(bkid), main = "raw", pch = ".", log = "xy")
```
```{r nkid-calib5, warning = FALSE, fig.cap = "Scatterplot for the badly biased `bkid` data: after calibration and transformation with vsn.", fig.small = TRUE}
plot(exprs(bfit), main = "vsn", pch = ".")
coef(bfit)[1,,]
```

Notice the change in the parameter $b$ of the red channel: it is now larger by about $\log(4)\approx 1.4$, and the shift parameter $a$ has also been adjusted.  


# Variance stabilisation without calibration{#sec:nocalib}

It is possible to force $\lambda_{si}=1$ and $\alpha_{si}=0$ for all $s$ and $i$ in Equation \@ref(eq:cal) by setting `vsn2`'s parameter `calib` to `none`. Hence, only the global variance stabilisation transformation \@ref(eq:vs) will be applied, but no column- or row-specific calibration.

Here, I show an example where this feature is used in conjunction with
quantile normalisation.

```{r vsnQ1, results = "hide", fig.cap = "Scatterplot between the red and green intensities of the array of the lymphoma dataset after quantile normalisation followed by VSN variance stabilisation without calibration. ", fig.small = TRUE} 
lym_q = normalizeQuantiles(exprs(lymphoma))
lym_qvsn = vsn2(lym_q, calib="none")
plot(exprs(lym_qvsn)[, 1:2], pch=".", main="lym_qvsn")
```
```{r vsnQ2, results = "hide", fig.cap = "Comparison of the values calculated in this section, for CH1 of the first array, to those of VSN variance stabilisation with affine calibration (`lym` was computed in Section \\@ref(sec:lymphoma)).", fig.small = TRUE} 
plot(exprs(lym)[,1], exprs(lym_qvsn)[, 1], 
  main="lym_qvsn vs lym", pch=".", 
  ylab="lym_qvsn[,1]", xlab="lym[,1]")
```

# Assessing the performance of VSN

VSN is a parameter estimation algorithm that fits the parameters for a
certain model. In order to see how good the estimator is, we can look
at bias, variance, sample size dependence, robustness against model
misspecificaton and outliers. This is done in the vignette
*Verifying and assessing the performance with simulated data*
that comes with this package.

Practically, the more interesting question is how different microarray
calibration and data transformation methods compare to each other.
Two such comparisons were made in reference [@Huber:2002:Bioinformatics], one
with a set of two-colour cDNA arrays, one with an Affymetrix genechip
dataset. Fold-change estimates from VSN led to
higher sensitivity and specificity in identifying differentially
expressed genes than a number of other methods.

A much more sophisticated and wider-scoped approach was taken by the
[Affycomp benchmark study](http://affycomp.biostat.jhsph.edu).  It uses two benchmark
datasets: a *Spike-In* dataset, in which a small number of cDNAs
was spiked in at known concentrations and over a wide range of
concentrations on top of a complex RNA background sample; and a
*Dilution* dataset, in which RNA samples from heart and brain
were combined in a number of dilutions and proportions.  The design of
the benchmark study, which has been open for anyone to submit their method, was
described in [@Affycomp].  A discussion of its results
was given in [@Affycomp2]. One of the results that emerged
was that VSN compares well with the background correction and
quantile normalization method of RMA; both methods place a high
emphasis on *precision* of the expression estimate, at the price
of a certain *bias* (see also Section \@ref(sec:shrink)). Another
result was that reporter-sequence specific effects (e.g., the effect
of GC content) play a large role in these data and that substantial
improvements can be achieved when they are taken into account
(something which VSN does not do).

Of course, the two datasets that were used in Affycomp were somewhat
artificial: they had fewer differentially expressed genes and 
were probably of higher quality than in most
real-life applications. And, naturally, in the meanwhile the
existence of this benchmark has led to the development of new
processing methods where a certain amount of overfitting may have
occured.

I would also like to note the interaction between
normalization/preprocessing and data quality. For data of high
quality, one can argue that any decent preprocessing method should
produce more or less the same results; differences arise when the data
are problematic, and when more or less successful measures may be
taken by preprocessing methods to correct these problems.

# VSN, shrinkage and background correction{#sec:shrink}

```{r calcshrink, echo = FALSE, results = "hide"}
log2.na = function(x){
  w = which(x>0)
  res = rep(as.numeric(NA), length(x))
  res[w] = log2(x[w])
  return(res)
}
  
fc = 0.5                    ## true fold change
x2 = seq(0.5, 15, by=0.5)   ## 'true values' in sample 1
x1 = x2/fc                  ## 'true values' in sample 2
m = s = numeric(length(x1))
n  = 10000
sa = 1
b  = 1
sb = 0.1
sdeta = 0.1
for(i in 1:length(x1)){
  z1 = sa*rnorm(n)+x1[i]*b*exp(sb*rnorm(n))
  z2 = sa*rnorm(n)+x2[i]*b*exp(sb*rnorm(n))
  if(i%%2==1) {
    q = log2.na(z1/z2)
    m[i] = mean(q, na.rm=TRUE)
    s[i] = sd(q, na.rm=TRUE)
  } else {
    h = (asinh(z1/(sa*b/sb))-asinh(z2/(sa*b/sb)))/log(2)
    m[i] = mean(h)
    s[i] = sd(h)
  }
}

colq = c("black", "blue")
lty  = 1
pch  = c(20,18)
cex  = 1.4
lwd  = 2
``` 

```{r shrink, results = "hide", echo = FALSE, fig.cap = "The shrinkage property of the generalised log-ratio.  Blue diamonds and error bars correspond to mean and standard deviation of the generalised log-ratio $h$, as obtained from VSN, and black dots and error bars to the standard log-ratio $q$ (both base 2).", fig.small = TRUE}
mai = par("mai"); mai[3]=0; par(mai = mai)
plot(x2, m, ylim=c(-2, 3.5), type="n", xlab=expression(x[2]), ylab="")
abline(h=-log2(fc), col="red", lwd=lwd, lty=1)
abline(h=0, col="black", lwd=1, lty=2)
points(x2, m, pch=pch, col=colq, cex=cex)
segments(x2, m-2*s, x2, m+2*s, lwd=lwd, col=colq, lty=lty)
legend(8.75, -0.1, c("q","h"), col=colq, pch=pch, lty=lty, lwd=lwd)
```

```{r figgraph, results = "hide", echo = FALSE, fig.cap = "Graphs of the functions. $y=\\log_2(x)$, $y=$glog$_2(x,c)=\\log_2(x+\\sqrt{x^2+c^2})-1$, and $y=\\log_2(x+x_{\\text{off}})$, where $c=50$ and $x_{\\text{off}}=50$.", fig.height = 5, fig.small = TRUE}
par(mai = c(0.8, 0.6, 0.01, 0.01))
x = seq(-70.5, 170.5, by=1)
cols = c("black", "blue", "darkgrey")
xoff = cc = 50
ymat = cbind(log2.na(x), log2( (x+sqrt(x^2+cc^2))/2 ), log2.na(x+xoff))
ylim = range(ymat, na.rm=TRUE)
matplot(x, ymat, 
        lty=c(1,1,2), col=cols, lwd=3, type="l", ylim=ylim, ylab="")
abline(v = 0, col = "#80808080", lty = 2)
legend(x = par("usr")[2], y = par("usr")[3], 
       legend = c(expression(y = log[2](x)), expression(y = glog[2](x,c)), expression(y = log[2](x+x[off]))), 
       fill=cols, density = c(NA, NA, 50), xjust=1.1, yjust=-0.1)
```

Generalised log-ratios can be viewed as a *shrinkage estimator*: 
for low intensities either in the numerator and denominator,
they are smaller in absolute value than the standard log-ratios,
whereas for large intensities, they become equal.
Their advantage is that they do not suffer from the variance
divergence of the standard log-ratios at small intensities: they remain
well-defined and have limited variance when the data come close to zero or
even become negative. 

An illustration is shown in Figure \@ref(fig:shrink).  Data were
generated from the additive-multiplicative error
model [@RockeDurbin2001,@Huber:2003:SAGMB,@Huber:2004:Encyc].  The
horizontal line corresponds to the true $\log_2$-ratio $1$
(corresponding to a factor of 2).  For intensities $x_2$ that are
larger than about ten times the additive noise level $\sigma_a$,
generalised log-ratio $h$ and standard log-ratio $q$ coincide.  For
smaller intensities, we can see a *variance-bias trade-off*:
$q$ has almost no bias, but a huge variance, thus an estimate of the
fold change based on a limited set of data can be arbitrarily
off. In contrast, $h$ keeps a constant
variance -- at the price of systematically underestimating the true
fold change. This is the main argument for using a variance stabilising 
transformation.

Note that there is also some bias in the behaviour of $q$ for small $x_2$, particularly at
$x_2=0.5$. This results from the occurence of negative values in the
data, which are discarded from the sampling when the (log-)ratio is
computed. 

Please consult the references for more on the
mathematical background [@Huber:2002:Bioinformatics,@Huber:2003:HSG,@Huber:2003:SAGMB].

It is possible to give a Bayesian interpretation: our prior
assumption is the conservative one of no differential expression.
Evidence from a feature with high overall intensity is taken strongly,
and the posterior results in an estimate close to the empirical
intensity ratio. Evidence from features with low intensity is
downweighted, and the posterior is still strongly influenced by the prior.

# Quality assessment{#sec:qc}

Quality problems can often be associated with physical parameters of
the manufacturing or experimental process. Let us look a bit closer at
the `lymphoma` data. Recall that M is the
`r nrow(M)` times `r ncol(M)` matrix of generalized log-ratios
and `A` a matrix of the same size with the average
glog$_2$-transformed intensities. The dataframe
`arrayGeometry` (from Section \@ref(sec:strata))
contains, for each array feature, the
identifier of the print-tip by which it was spotted and the row and
column within the print-tip sector. Figure \@ref(fig:lymbox)
shows the boxplots of A values of array 
CLL-13 stratified by row.

```{r lymbox, fig.cap = "Boxplot of A values of array CLL-13 stratified by within-sector row. The features in rows 22 and 23 are all very dim."}
colours = hsv(seq(0,1, length = nsr), 0.6, 1)
j = "CLL-13"
boxplot(A[, j] ~ arrayGeometry$spotrow, col = colours, main = j, ylab = "A", xlab = "spotrow")
``` 

You may want to explore similar boxplots for other stratifying factors
such as column within print-tip sector or print-tip sector and look at
these plots for the other arrays as well.

In Figure \@ref(fig:lymbox), we see that the features in rows 22 and
23 are all very dim.  If we now look at these data in the $M$-$A$-plot
(Figure \@(fig:lymquscp)), we see that these features not only have
low $A$-values, but fall systematically away from the $M=0$ line.

```{r lymquscp, fig.cap = "MA-plot of the data from array `r j`. Dots coloured in orange are from rows 22 and 23. A possible explanation may be as follows (although I do not know for sure that this is the right explanation): The PCR product (cDNA) that is spotted on these arrays is put on by a print head that sucks cDNA out of microtitre plates and deposits them in spots one after another, row by row. If the content of one plate is faulty, this results in a set of subsequent spots that are faulty. Because the 16 print-tip sectors are spotted in parallel, this affects all sectors in the same way.", fig.small = TRUE}
plot(A[,j], M[,j], pch = 16, cex = 0.3,
  col = ifelse(arrayGeometry$spotrow%in%(22:23), "orange", "black"))
abline(h = 0, col = "blue")
``` 

Hence, in a naive analysis
the data from these features would be interpreted as contributing 
evidence for differential expression, while they are more 
likely just the result of a quality problem. 
So what can we do? There are some options:

- Flag the data of the affected features as unreliable and set them
aside from the subsequent analysis.
- Use a more complex, stratified normalisation method that takes
into account the different row behaviours, for example,
VSN with strata (see Section \@ref(sec:strata)).
- It has also been proposed to address this type of problem by using 
a non-linear regression on the $A$-values, for example the 
`loess` normalization of reference [@YeeNorm]
that simply squeezes the $M$-$A$-plot to force the centre of the 
distribution of $M$ to lie at 0 along the whole $A$-range.


An advantage of Option 3 is that it works without knowing the real
underlying stratifying factor.  However, it assumes that the
stratifying factor is strongly confounded with $A$, and that biases
that it causes can be removed through a regression on $A$.

In the current example, if we believe that the real underlying
stratifying factor is indeed row within sector, this assumption means
that (i) few of the data points from rows 22 and 23 have high
$A$-values, and that (ii) almost all data points with very low $A$
values are from these rows; while (i) appears tenable, (ii) is
definitely not the case.


## Stratifying factors such as print-tip, PCR plate, reporter-sequence

By default, the VSN method assumes that the measured 
signal $y_{ik}$ increases, to sufficient approximation, proportionally 
to the mRNA abundance $c_{ik}$ of gene $k$ on array $i$ (or 
in colour channel $i$):

\begin{equation}
y_{ik}\approx \alpha_i + \lambda_i \lambda_k c_{ik}.
(\#eq:assumption1)
\end{equation}

For a series of $d$ single-colour arrays, $i=1,\ldots,d$, 
and the different factors 
$\lambda_i$ reflect the different initial amounts of sample mRNA or different
overall reverse transcription, hybridisation and detection efficiencies. 
The feature affinity $\lambda_k$ contains factors that affect
all measurements with feature $k$ in the same manner, such as
sequence-specific labelling 
efficiency. The $\lambda_k$ are assumed to be the same across all arrays.
There can be a non-zero overall offset $\alpha_i$.
For a two-colour cDNA array, $i=1,2$, and the $\lambda_i$ take into account
the different overall efficiencies of the two dyes^[ 
It has been reported that for some genes the dye bias is different from 
gene to gene, such that the proportionality factor does not simply factorise 
as in \@ref(eq:assumption1). As long as this only occurs sporadically, this 
will not have much effect on the estimation of the calibration and 
variance stabilisation parameters. Further, by using an appropriate 
experimental design such as colour-swap or reference design, the effects of 
gene-specific dye biases on subsequent analyses can be reduced.].

Equation \@ref(eq:assumption1) can be generalised to

\begin{equation}
y_{ik}\approx \alpha_{is} + \lambda_{is} \lambda_k c_{ik}.
(\#eq:assumption2)
\end{equation}

that is, the background term $\alpha_{is}$ and the gain factor $\lambda_{is}$ can
be different for different groups $s$ of features on an array.  The
VSN methods allows for this option by using the `strata`
argument of the function `vsn2`.  We have seen an example above
where this could be useful.  For Affymetrix genechips, one can find
systematic dependences of the affinities $\lambda_{is}$ and the background
terms $\alpha_{is}$ on the reporter sequence.

Nevertheless, there are situations in which either 
assumption \@ref(eq:assumption1) or \@ref(eq:assumption2) 
is violated, and these include:

Saturation
: The biochemical reactions and/or the photodetection 
can be run in such a manner that saturation effects occur.
It may be possible to rescue such data by using non-linear transformations. 
Alternatively, it is recommended that the experimental parameters are 
chosen to avoid saturation.

Batch effects
: The feature affinities $\lambda_k$ may differ between
different manufacturing batches of arrays due, e.g., to different
qualities of DNA amplification or printing. VSN cannot be
used to simultaneously calibrate and transform data from different
batches.

How to reliably diagnose and deal with such violations is beyond the scope 
of this vignette; see the references for more [@Huber:2003:HSG,@Dudoit578]. 

## Most genes unchanged assumption{#sec:mostunch}

With respect to the VSN model fitting, data from 
differentially transcribed genes can act as outliers 
(but they do not necessarily need to do so in all cases).
The maximal number of outliers that do not gravely affect the model
fitting is controlled by the parameter `lts.quantile`. 
Its default value is 0.9, which allows for 10% outliers. The value of
`lts.quantile` can be reduced down to 0.5, which allows for up
to 50% outliers. The maximal value is 1, which results in a
least-sum-of-squares estimation that does not allow for any outliers.

So why is this parameter `lts.quantile` user-definable and why 
don't we just always use the most "robust" value of 0.5?
The answer is that the precision of the estimated VSN 
parameters is better the more data points go into the estimates, and
this may especially be an issue for arrays with a small number of 
features^[more precisely, number of features per stratum].
So if you are confident that the number of outliers is not that large,
using a high value of `lts.quantile` can be justified.

There has been confusion on the role of the ``most
genes unchanged assumption'', which presumes that only a minority of
genes on the arrays is detectably differentially transcribed across
the experiments. This assumption is a sufficient condition for
there being only a small number of outliers, and these would not gravely
affect the VSN model parameter estimation. However, it is
not a necessary condition: the parameter estimates and the
resulting normalised data may still be useful if the
assumption does not hold, but if the effects of the data from
differentially transcribed genes balance out.

# Acknowledgements

I acknowledge helpful comments and feedback from Anja von Heydebreck, Richard Bourgon,
Martin Vingron, Ulrich Mansmann and Robert Gentleman.  The active development of the
`r Biocpkg("vsn")` package was from 2001 to 2003, and most of this document was written
during that time (although the current layout is more recent).


# Session info {.unnumbered}

```{r sessionInfo}
sessionInfo()
```

# References

%\VignetteIndexEntry{vsn - verifying and assessing the performance with simulated data}
%\VignetteDepends{Biobase,vsn}
%\VignetteKeywords{Expression Analysis}
%\VignettePackage{vsn}

\documentclass[11pt]{article}
\usepackage[margin=2cm,noheadfoot]{geometry}

%------------------------------------------------------------
% newcommand
%------------------------------------------------------------
\newcommand{\Rfunction}[1]{\texttt{#1}}
\newcommand{\Robject}[1]{\texttt{#1}}
\newcommand{\Rpackage}[1]{\textit{#1}}
\newcommand{\Rclass}[1]{\textit{#1}}

\SweaveOpts{keep.source=TRUE} 

\begin{document}
%------------------------------------------------------------------------
\title{Performance assessment of \Rpackage{vsn} with simulated data}
%------------------------------------------------------------------------
\author{Wolfgang Huber}
\maketitle
\tableofcontents

\section{Overview}
The purpose of this vignette is to assess that the software in
\Rpackage{vsn} does what it is intended do, and in particular, 
to assess the performance of the parameter estimation on simulated 
data where the true parameters are known.

There are two functions \Rfunction{sagmbSimulateData} and
\Rfunction{sagmbAssess} that can be used to generate simulated data and assess
the difference between the 'true' and 'estimated' data calibration and
transformation by \Rpackage{vsn}. This vignette demonstrates some examples. 
Please refer to reference~\cite{HuberSAGMB2003} for more detail on the
simulation model,  the assessment strategy and a comprehensive
suite of assessments with respect to the number of features \Robject{n}, the
number of arrays \Robject{d}, the fraction of differentially expressed genes
\Robject{de}, and the fraction of up-regulated genes \Robject{up}.
%

\section{Helper functions used in this document}
This section is given just for completeness -- for the results, you can skip this and 
go to Section~\ref{sec:n}.

<<setup, results=hide>>=
library("vsn")
set.seed(0xabcd)
## options(error=recover, warn=2)
@ 

The function \Rfunction{sim} computes simulated data using the
function \Rfunction{sagmbSimulateData} from the \Rpackage{vsn}
package, calls \Rfunction{vsn2} to fit the VSN model, and assesses
model fit using the function \Rfunction{sagmbSimulateData}.

<<sim, results=hide>>=
sim = function(..., lts.quantile=1, nrrep=30L) {
  callpar = list(...)
  ll      = listLen(callpar)
  stopifnot(ll[1]>=1, all(ll[-1]==1))
  res  = matrix(1, nrow=nrrep, ncol=ll[1])
 
  ## default parameters
  simpar = append(callpar, 
           list(n=4096L, d=2L, de=0, up=0.5, nrstrata=1L, miss=0, log2scale=TRUE)) 
  simpar = simpar[!duplicated(names(simpar))]

  for (i in 1:ll[1]) {
    simpar[[1]] = callpar[[1]][i]
    for (r in 1:nrrep) {
      sim = do.call("sagmbSimulateData", simpar)
      ny  = vsn2(sim$y, strata=factor(sim$strata), lts.quantile=lts.quantile, verbose=!TRUE)
      res[r, i] = sagmbAssess(exprs(ny), sim)  
      if(!TRUE){
        cat(paste(sprintf("%6g", signif(sim$coefficients, 3), collapse=" ")), "")
        cat(sprintf("i=%d, r=%d: %g\n", i, r, signif(res[r,i], 3)))
        plot(sim$y, pch="."); abline(a=0, b=1, col="orange", main=paste(i,r))
        if(res[r, i]>0.02) browser()
      }
   } ## for r
  } ## for i
  return(res)
} 
@ 

Here some functions to automate the formating of the plots that are used in 
the following.

<<onePlot>>=
onePlot = function(n, res, log="xy", ...) {
  matplot(n, t(res), pch=20, log=log, ylab='r.m.s. error', col="orange", 
          xlab=deparse(substitute(n)), ...)
  lines(n, colMeans(res), col="blue")
}

twoPlot = function(n, rl) {
  par(mfrow=c(1,2))
  ylim=range(unlist(rl))
  for(i in seq(along=rl)) {
    x = if(is.list(n)) n[[i]] else n
    matplot(x, t(rl[[i]]), pch=20, ylab='r.m.s. error', xlab=deparse(substitute(n)),
            log="y", ylim=ylim, col="orange", main=names(rl)[i])
    lines(x, colMeans(rl[[i]]), col="blue")
  }
}

makeFig = function(name, width, height, expr) {
  pdfname = paste(name, "pdf", sep=".")
  pdf(file=pdfname, width=4*width, height=4*height)
  expr
  dev.off()
  invisible(pdfname)
}
@

%--------------------------------------------------
\section{Number of features $n$}\label{sec:n}
%--------------------------------------------------
Fig.~\ref{fign} shows the estimation error for the transformation (i.\,e.\
the root mean squared difference between true and estimated transformed data)
as a function of the number of features $n$. If \Rfunction{vsn} works correctly,
the estimation error should decrease roughly as $n^{-1/2}$.
%
<<fign1, results=hide>>=
n = 1000*2^seq(-2, 5)
makeFig("fign1", 1, 1, {
  res = sim(n=n)
  onePlot(n, res, main="n: one stratum")
})
@
<<fign2, results=hide>>=
n = 1000*2^seq(0, 5)
makeFig("fign2", 1, 1, {
  res = sim(n=n, nrstrata=8)
  onePlot(n, res, main="n: 8 strata")
})
@
%
\begin{figure}[tbp]
\begin{center}
\includegraphics[width=0.49\textwidth]{fign1}
\includegraphics[width=0.49\textwidth]{fign2}
\caption{\label{fign}%
Estimation error as a function of the number of features $n$. 
If \Rfunction{vsn} works correctly,
the estimation error should decrease roughly as $n^{-1/2}$.}
\end{center}
\end{figure}

%--------------------------------------------------
\section{Number of samples $d$}\label{sec:d}
%--------------------------------------------------
Fig.~\ref{figds}a shows the estimation error as a function of the number of
samples $d$. This curve is essentially flat. This is because the number of parameters that need to be
estimated is proportional to $d$, so the "number of data points per parameter"
is constant in this plot (in contrast to Fig.~\ref{fign}).
%
<<figd, results=hide>>=
makeFig("figd", 1, 1, {
  d = 2^seq(1, 5)
  res = sim(d=d)
  onePlot(d, res, main="a) d")
})
@
%--------------------------------------------------
\section{Number of strata}\label{sec:nrstrata}
%--------------------------------------------------
In Fig.~\ref{figds}b, we see the estimation error as a function of the number of strata. It
should increase, since for each stratum, we need to estimate separate
parameters, and if the overall number of features does not change, more strata
means less data per parameter.
%
<<fignrstrata, results=hide>>=
makeFig("fignrstrata", 1, 1, {
nrstrata = 2^seq(0, 4)
res = sim(nrstrata=nrstrata)
onePlot(nrstrata, res, main="b) nrstrata")
})
@
%
\begin{figure}[tbp]
\begin{center}
\includegraphics[width=0.49\textwidth]{figd}
\includegraphics[width=0.49\textwidth]{fignrstrata}
\caption{\label{figds}%
Estimation error as a function of (a) the number of samples
and (b) the number of strata. See Sections~\ref{sec:d} and \ref{sec:nrstrata}.}
\end{center}
\end{figure}

%-----------------------------------------------------
\section{Differentially expressed genes}\label{sec:de}
%------------------------------------------------------
In the following code, \Robject{de} is the fraction of differentially expressed genes.
We run the simulation both with \Rfunction{vsn2}'s default setting \Robject{lts.quantile=0.9} and 
the more robust \Robject{lts.quantile=0.5}.
The reason why \Robject{lts.quantile=0.5} is not the default is that 
the estimator with \Robject{lts.quantile=0.9} 
is more efficient (more precise with less data) \textit{if} the fraction of 
differentially expressed genes is not that large. See Figure~\ref{figdiff}.
%
<<figdiff, results=hide>>=
makeFig("figdiff", 2, 1, {
de  = (0:6)/10
res1 = sim(de=de, nrstrata=2, lts.quantile=0.9)
res2 = sim(de=de, nrstrata=2, lts.quantile=0.5)
twoPlot(de, list("de, lts.quantile=0.9"=res1, "de, lts.quantile=0.5"=res2))
})
@
%
In the next code chunk, \Robject{up} is the fraction of up-regulated
genes among the differentially expressed genes.
%
<<figup, results=hide>>=
makeFig("figup", 2, 1, {
up  = (0:8)/8
res1 = sim(up=up, nrstrata=2, de=0.2)
res2 = sim(up=up, nrstrata=2, de=0.2, lts.quantile=0.5)
twoPlot(up, list("a) up, lts.quantile=0.9"=res1, "b) up, lts.quantile=0.5"=res2))
})
@
%
\begin{figure}[tbp]
\begin{center}
\includegraphics[width=\textwidth]{figdiff}
\caption{\label{figdiff}%
Estimation error as a function of the number of differentially expressed genes,
for two different settings of \Robject{lts.quantile}. Note how a) is better
for small values auf \Robject{de}, but becomes worse for larger
values of \Robject{de}. See Section~\ref{sec:de}.}
\end{center}
\end{figure}

\begin{figure}[tbp]
\begin{center}
\includegraphics[width=\textwidth]{figup}
\caption{\label{figup}%
Estimation error as a function of the fraction of up-regulated genes, 
for two different settings of \Robject{lts.quantile};
see Section~\ref{sec:de}.}
\end{center}
\end{figure}

%-----------------------------------------------------
\section{Missing values}\label{sec:miss}
%------------------------------------------------------
In this Section, we check the impact of missing values on the performance 
of the estimator. \Robject{miss} is the fraction of missing values in
the overall 
%
<<figmiss, results=hide>>=
makeFig("figmiss", 2, 1, {
miss1 = seq(0, 0.5, length=6)
res1 = sim(miss=miss1, d=8)
miss2 = seq(0, 0.1, length=6)
res2 = sim(miss=miss2, d=2)
twoPlot(list(miss1, miss2), list("fraction NA (d=8)"=res1, "b) fraction NA (d=2)"=res2))
})
@ 
%
\begin{figure}[tbp]
\begin{center}
\includegraphics[width=\textwidth]{figmiss}
\caption{\label{figmiss}%
Estimation error as a function of the fraction of missing data points,
see Section~\ref{sec:miss}.}
\end{center}
\end{figure}

%-----------------------------------------------------
\section{Incremental normalization}\label{sec:inc}
%------------------------------------------------------
First, let's simulate a dataset with 10000 features, 12 arrays, 
and no differentially expressed genes (in order to be able to look
at the ML estimates rather than their robustified modifications).
%
<<incr1>>=
dat = sagmbSimulateData(n=10000, d=12, de=0, nrstrata=1, miss=0, log2scale=TRUE)
v  = new("vsn", mu=dat$mu, sigsq=dat$sigsq)
fit = vsn2(dat$y, lts.quantile=1, verbose=FALSE)
@ 
%
\Robject{fit} contains the maximum profile likelihood estimate of 
the \Rpackage{vsn} model.
Then we use the \emph{incremental mode} of \Rpackage{vsn} to estimate,
in turn for each array individually, the parameters. 
The results are shown in Figure~\ref{figincr}.
%
<<incr2>>=
parRef = array(as.numeric(NA), dim=dim(coef(fit)))
for(j in seq_len(ncol(dat$y))) {
  vj = vsn2(dat$y[,j], reference=v, lts.quantile=1, verbose=FALSE)
  parRef[,j,] = coef(vj)
}
@ 
%
<<figincr, results=hide>>=
makeFig("figincr", 2, 2, {
par(mfcol=c(2,2))
for(k in 1:2) {
  plot(dat$coefficients[1,,k], parRef[1,,k], pch=16, xlab="True", ylab="Reference fit", 
    main=c("offset", "factor")[k])
  abline(a=0, b=1, col="orange")
  plot(coefficients(fit)[1,,k], parRef[1,,k], pch=16, xlab="Profile Likelihood fit", ylab="Reference fit", 
    main=c("offset", "factor")[k])
  abline(a=0, b=1, col="orange")
}
})
@ 
%
\begin{figure}[tbp]
\begin{center}
\includegraphics[width=\textwidth]{figincr}
\caption{\label{figincr}%
Comparison of parameters fitted from incremental normalisation ($y$-axis) with
true parameters ($x$-axis, upper row) and with
parameters fitted from joint profile-likelihood normalisation ($x$-axis, lower row);
see Section~\ref{sec:inc}.}
\end{center}
\end{figure}

%-----------------------------------------------------
%\section{Negligible additive background}\label{sec:multionly}
%------------------------------------------------------
%Datasets in which the multiplicative noise component dominated over the additive component, 
%and hence in which the usual logarithmic transformation is appropriate, were problematic 
%for previous versions of \Rpackage{vsn}.

%---------------------------------------------------------
% SessionInfo
%---------------------------------------------------------
\begin{table*}[tbp]
\begin{minipage}{\textwidth}
<<sessionInfo, results=tex, print=TRUE>>=
toLatex(sessionInfo())
@ 
\end{minipage}
\caption{\label{tab:sessioninfo}%
The output of \Rfunction{sessionInfo} on the build system 
after running this vignette.}
\end{table*}


\begin{thebibliography}{10}
%-----------------------------------------------------

\bibitem{HuberSAGMB2003}
W. Huber, A. von Heydebreck, H. {S\"ultmann}, A. Poustka, and M. Vingron.
\newblock Parameter estimation for the calibration and variance stabilization 
of microarray data.
\newblock \textit{Statistical Applications in Genetics and Molecular Biology}, 
Vol. 2: No. 1, Article 3, 2003. 
http://www.bepress.com/sagmb/vol2/iss1/art3

\end{thebibliography}

\end{document}

%\VignetteIndexEntry{vsn - verifying and assessing the performance with simulated data}
%\VignetteDepends{Biobase,vsn}
%\VignetteKeywords{Expression Analysis}
%\VignettePackage{vsn}

\documentclass[11pt]{article}
\usepackage[margin=2.5cm,noheadfoot]{geometry}

%------------------------------------------------------------
% newcommand
%------------------------------------------------------------
\newcommand{\Rfunction}[1]{\texttt{#1}}
\newcommand{\Robject}[1]{\texttt{#1}}
\newcommand{\Rpackage}[1]{\textit{#1}}
\newcommand{\Rfunarg}[1]{\textit{#1}}
\newcommand{\Rclass}[1]{\textit{#1}}

\begin{document}
\SweaveOpts{eps=false}

%------------------------------------------------------------------------
\title{How to assess the accuracy of vsn with simulated data}
%------------------------------------------------------------------------
\author{Wolfgang Huber}
\maketitle
<<no.nonsense,eval=FALSE,echo=FALSE,results=hide>>=
q(save="no")
@

The purpose of this vignette is to demonstrate that the software in
\Rpackage{vsn} actually does what it is supposed to do according to the
mathematical theory. And to see \textit{how fast} (or slow, depending on your
point of view) and \textit{how accurately} it does that.

There are two functions \Rfunction{sagmbSimulateData} and
\Rfunction{sagmbAssess} that can be used to generate simulated data and assess
the difference between the 'true' and 'estimated' data calibration and
transformation by \Rpackage{vsn}. This vignette demonstrates some examples. 
Reference~\cite{HuberSAGMB2003} describes in more detail (i) the
simulation model, (ii) the assessment strategy, and (iii) a comprehensive
suite of assessments with respect to the number of probes \Robject{n}, the
number of arrays \Robject{d}, the fraction of differentially expressed genes
\Robject{de}, and the fraction of up-regulated genes \Robject{up}.
<<setup,results=hide>>=
library(vsn)
set.seed(1)

sim <- function(..., nrrep=16) {
  callpar <- list(...)
  ll      <- sapply(callpar, length)
  stopifnot(ll[1]>1, all(ll[-1]==1))
  res  <- matrix(NA, nrow=nrrep, ncol=ll[1])
  simpar <- append(callpar, list(n=4096, d=2, de=0, up=0.5, nrstrata=1)) ## default parameters
  simpar <- simpar[!duplicated(names(simpar))]
  for (i in 1:ll[1]) {
    simpar[[1]] <- callpar[[1]][i]
    for (r in 1:nrrep) {
      sim <- do.call("sagmbSimulateData", simpar)
      ny  <- vsn(sim$y, strata=sim$strata, verbose=FALSE)
      ## plot(exprs(ny)[, 1:2], pch=16, col=1+preproc(description(ny))$vsnTrimSelection)
      res[r, i] <- sagmbAssess(exprs(ny), sim)
    }
  }
  return(res)
}
@

The following plot shows the estimation error for the transformation (i.\,e.\
the root mean squared difference between true and estimated transformed data)
as a function of the number of genes $n$. If \Rfunction{vsn} works correctly,
the estimation error should decrease roughly as $n^{-1/2}$.
<<n1, fig=TRUE>>=
n   <- 1000*2^seq(-2, 5)
res <- sim(n=n)
matplot(n, t(res), pch=20, log="xy", col="#909090", main="n")
lines(n, colMeans(res), col="blue")
@

\newpage
The same with 8 strata:
<<n2, fig=TRUE>>=
n   <- 1000*2^seq(1, 5)
res <- sim(n=n, nrstrata=8)
matplot(n, t(res), pch=20, log="xy", col="#909090", main="n (8 strata)")
lines(n, colMeans(res), col="blue")
@

\newpage
The following plot shows the estimation error as a function of the number of
samples $d$. Initially, it also decreases slightly with $d$, but eventually
reaches a plateau. This is because the number of parameters that need to be
estimated is proportional to $d$, so the "number of data points per parameter"
is constant in this plot (above, it was increasing proportionally to $n$).
<<d, fig=TRUE>>=
d   <- 2^seq(1, 5)
res <- sim(d=d)
matplot(d, t(res), pch=20, log="xy", col="#909090", main="d")
lines(d, colMeans(res), col="blue")
@

\newpage
Here, we see the estimation error as a function of the number of strata. It
should increase, since for each stratum, we need to estimate separate
parameters, and if the overall number of probes does not change, more strata
means less and less data per parameters.
<<nrstrata, fig=TRUE>>=
nrstrata <- 2^seq(0, 4)
res <- sim(nrstrata=nrstrata)
matplot(nrstrata, t(res), pch=20, log="xy", col="#909090", main="nrstrata")
lines(nrstrata, colMeans(res), col="blue")
@

\newpage
\Robject{de} is the fraction of differentially expressed genes.
<<de, fig=TRUE>>=
de  <- (0:6)/10
res <- sim(de=de)
matplot(de, t(res), pch=20, col="#909090", main="de")
lines(de, colMeans(res), col="blue")
@

\newpage
\Robject{up} is the fraction of up-regulated genes among the differentially
expressed genes.  The best results are obtained for \Robject{up}$\approx0.5$,
while the estimation goes up the more unbalanced the situation becomes.
<<up, fig=TRUE>>=
up  <- (0:8)/8
res <- sim(up=up, de=0.2)
matplot(up, t(res), pch=20, col="#909090", main="up")
lines(up, colMeans(res), col="blue")
@
\begin{thebibliography}{10}

\bibitem{HuberSAGMB2003}
W. Huber, A. von Heydebreck, H. {S\"ultmann}, A. Poustka, and M. Vingron.
\newblock Parameter estimation for the calibration and variance stabilization 
of microarray data.
\newblock \textit{Statistical Applications in Genetics and Molecular Biology}, 
Vol. 2: No. 1, Article 3, 2003. 
http://www.bepress.com/sagmb/vol2/iss1/art3

\end{thebibliography}

\end{document}

\name{vsn-package}
\alias{vsn-package}
%% \alias{vsn}
\docType{package}
\title{vsn}
\description{vsn}
\details{
  The main function of the package is \code{\link{vsn2}}.
  Interesting for its applications are also
  \code{predict} and the wrapper function \code{\link{justvsn}}.
  
  \code{\link{vsn2}} can be applied to objects of class
  \code{\link[Biobase:class.ExpressionSet]{ExpressionSet}},
  \code{\link[Biobase:class.NChannelSet]{NChannelSet}},
  \code{\link[affy:AffyBatch-class]{AffyBatch}} (from the \code{affy} package) and
  \code{\link[limma:rglist]{RGList}} (from the \code{limma} package),
  \code{matrix} and \code{vector}. It returns an object of class
  \code{\linkS4class{vsn}}, which contains the results of fitting the
  \code{vsn} model to the data.

  The most common use case is that you will want to construct a new
  data object with the vsn-normalized data whose class is the same as
  that of the input data and which preserves the metadata. This can be
  achieved by

  \preformatted{
    fit = vsn2(x, ...)
    nx = predict(fit, newdata=x)
  }

  To simplify this, there exists also a simple wrapper
  \code{\link{justvsn}}.
}

\author{
   Wolfgang Huber
}
\keyword{package}

\name{justvsn}
\alias{justvsn}
\alias{vsnrma}

\title{Wrapper functions for vsn}
\description{\code{justvsn} is equivalent to calling
\preformatted{
  fit = vsn2(x, ...)
  nx = predict(fit, newdata=x, useDataInFit = TRUE)
}
\code{vsnrma} is a wrapper around \code{\link{vsn2}} and \code{\link[affy:rma]{rma}}.    
}

\usage{
justvsn(x, ...)
vsnrma(x, ...)
}

\arguments{
  \item{x}{For \code{justvsn}, any kind of object for which
    \code{\link{vsn2}} methods exist.
    For \code{vsnrma}, an
    \code{\link[affy:AffyBatch-class]{AffyBatch}}.}
  \item{\dots}{Further arguments that get passed on to \code{\link{vsn2}}.}
}

\details{
  \code{vsnrma} does probe-wise
  background correction and between-array normalization by calling
  \code{\link{vsn2}} on the perfect match (PM) values only. Probeset
  summaries are calculated with the medianpolish algorithm of
  \code{\link[affy:rma]{rma}}. 
}

\value{\code{justvsn} returns the vsn-normalised intensities in
  an object generally of the same class as its first
  argument (see the man page of \code{\link[vsn:vsn2trsf]{predict}} for
  details). It preserves the metadata.
  
  \code{vsnrma} returns an \code{\link[Biobase:class.ExpressionSet]{ExpressionSet}}.
}
  
\seealso{
  \code{\link{vsn2}}
}

\author{Wolfgang Huber}

\examples{
##--------------------------------------------------
## use "vsn2" to produce a "vsn" object
##--------------------------------------------------
data("kidney")
fit = vsn2(kidney)
nkid = predict(fit, newdata=kidney)

##--------------------------------------------------
## justvsn on ExpressionSet
##--------------------------------------------------
nkid2 = justvsn(kidney)
stopifnot(identical(exprs(nkid), exprs(nkid2)))

##--------------------------------------------------
## justvsn on RGList
##--------------------------------------------------
rg = new("RGList", list(R=exprs(kidney)[,1,drop=FALSE], G=exprs(kidney)[,2,drop=FALSE]))
erge = justvsn(rg)
}

\name{sagmbSimulateData}
\alias{sagmbSimulateData}
\alias{sagmbAssess}
\title{Simulate data and assess vsn's parameter estimation}
\description{Functions to validate and assess the performance of vsn
  through simulation of data.}
\usage{
sagmbSimulateData(n=8064, d=2, de=0, up=0.5, nrstrata=1,  miss=0, log2scale=FALSE)
sagmbAssess(h1, sim)
}
\arguments{
  \item{n}{Numeric. Number of probes (rows).}
  \item{d}{Numeric. Number of arrays (columns).}
  \item{de}{Numeric. Fraction of differentially expressed genes.}
  \item{up}{Numeric. Fraction of up-regulated genes among the
    differentially expressed genes.}
  \item{nrstrata}{Numeric. Number of probe strata.}
  \item{miss}{Numeric. Fraction of data points that is randomly sampled
    and set to \code{NA}.}
  \item{log2scale}{Logical. If \code{TRUE}, glog on base 2 is used, if \code{FALSE},
    (the default), then base e.}
  \item{h1}{Matrix. Calibrated and transformed data, according, e.g., to
    vsn}
  \item{sim}{List. The output of a previous call to
    \code{sagmbSimulateData}, see Value}
}

\value{
  For \code{sagmbSimulateData}, a list with four components:
  \code{hy}, an \code{n x d} matrix with the true (=simulated)
  calibrated, transformed data;
  \code{y}, an \code{n x d} matrix with the simulated
  uncalibrated raw data - this is intended to be fed into
  \code{\link{vsn2}};
  \code{is.de}, a logical vector of length \code{n}, specifying
  which probes are simulated to be differentially expressed.
  \code{strata}, a factor of length \code{n}.

  For \code{sagmbSimulateData}, a number: the root mean squared
  difference between true and estimated transformed data.
}

\details{Please see the vignette.}

\references{Wolfgang Huber, Anja von Heydebreck, Holger Sueltmann,
  Annemarie Poustka, and Martin Vingron (2003)
  "Parameter estimation for the calibration and variance stabilization
  of microarray data",
  Statistical Applications in Genetics and Molecular Biology:
  Vol. 2: No. 1, Article 3.
  http://www.bepress.com/sagmb/vol2/iss1/art3}

\author{Wolfgang Huber}

\examples{
  sim <- sagmbSimulateData(nrstrata = 4)
  ny  <- vsn2(sim$y, strata = sim$strata)
  res <- sagmbAssess(exprs(ny), sim)
  res
}

\name{vsnInput}
\docType{class}
\alias{class:vsnInput}
\alias{vsnInput}
\alias{vsnInput-class}

\alias{[,vsnInput-method} 
\alias{dim,vsnInput-method} 
\alias{nrow,vsnInput-method} 
\alias{ncol,vsnInput-method} 
\alias{show,vsnInput-method} 

\title{
  Class to contain input data and parameters for vsn functions
}

\description{
  Class to contain input data and parameters for vsn functions
}

\section{Creating Objects}{
  \code{new("vsnInput")}
}

\section{Slots}{
 \describe{
  \item{\code{x}:}{A numeric matrix with the input data.}
  \item{\code{reference}:}{An object of \code{\linkS4class{vsn}},
    typically this would have been
    obtained from a previous fit to a set of reference arrays (data).}
  \item{\code{strata}:}{A factor of length 0 or n. If its length is
    n, then its levels correspond to different normalization strata
    (see \code{\link{vsn2}}).}
  \item{\code{ordered}:}{Logical scalar; are the rows reordered so that
    the strata are contiguous.}
  \item{\code{lts.quantile}:}{Numeric scalar, see\code{\link{vsn2}}.}
  \item{\code{subsample}:}{Integer scalar, see\code{\link{vsn2}}.}
  \item{\code{verbose}:}{Logical scalar, see\code{\link{vsn2}}.}
  \item{\code{calib}}{Character of length 1, see manual page of \code{\link{vsn2}}.}
  \item{\code{pstart}:}{A 3D array of size (number of strata) x (number of
    columns of the data matrix) x 2. It contains the start parameters.}
  \item{\code{optimpar}:}{List with parameters for the numerical
    optimiser \code{L-BFGS-B}; see the manual page of \code{\link{vsn2}}.}
  }
}


\section{Methods}{
  \describe{
    \item{\code{[}}{Subset}
    \item{\code{dim}}{Get dimensions of data matrix.}
    \item{\code{nrow}}{Get number of rows of data matrix.}
    \item{\code{ncol}}{Get number of columns of data matrix.}
    \item{\code{show}}{Print a summary of the object}
}}

\author{Wolfgang Huber}

\seealso{
  \code{\link{vsn2}}
}

\keyword{classes}

\name{meanSdPlot}
\docType{methods}
\alias{meanSdPlot}
\alias{meanSdPlot-methods}
\alias{meanSdPlot,matrix-method}
\alias{meanSdPlot,ExpressionSet-method}
\alias{meanSdPlot,vsn-method}
\alias{meanSdPlot,MAList-method}
\title{Plot row standard deviations versus row means}

\description{Methods for objects of classes
  \code{\link{matrix}}, 
  \code{\link[Biobase:class.ExpressionSet]{ExpressionSet}},
  \code{\linkS4class{vsn}} and
  \code{\link[limma:malist]{MAList}}
  to plot row standard deviations versus row means.}
\usage{
meanSdPlot(x, 
           ranks = TRUE,
           xlab  = ifelse(ranks, "rank(mean)", "mean"),
           ylab  = "sd",
           pch,
           plot  = TRUE,
	   bins  = 50,
           \dots)} 
\arguments{
  \item{x}{An object of class
    \code{\link{matrix}}, 
    \code{\link[Biobase:class.ExpressionSet]{ExpressionSet}}, 
    \code{\linkS4class{vsn}} or
    \code{\link[limma:malist]{MAList}}.}
\item{ranks}{Logical, indicating whether the x-axis (means) should be plotted
  on the original scale (\code{FALSE}) or on the rank scale (\code{TRUE}). The latter
  distributes the data more evenly along the x-axis and allows a
  better visual assessment of the standard deviation as a function of 
  the mean.}
\item{xlab}{Character, label for the x-axis.}
\item{ylab}{Character, label for the y-axis.}
\item{pch}{Ignored - exists for backward compatibility.}
\item{plot}{Logical.  If \code{TRUE} (default), a plot is produced.
  Calling the function with \code{plot=FALSE} can be useful if only
  its return value is of interest.}
\item{bins}{Gets passed on to \code{\link[ggplot2:geom_hex]{geom_hex}}.}
\item{\dots}{Further arguments that get passed on to \code{\link[ggplot2:geom_hex]{geom_hex}}.}
}
\details{Standard deviation and mean are calculated row-wise from the
  expression matrix (in) \code{x}. The scatterplot of these versus each other
  allows you to visually verify whether there is a dependence of the standard
  deviation (or variance) on the mean.
  The red line depicts the running median estimator (window-width 10\%).
  If there is no variance-mean dependence, then the line should be approximately horizontal.
}
\value{
  A named list with five components: its elements \code{px} and
  \code{py} are the x- and y-coordinates of the individual data points
  in the plot; its first and second element are the x-coordinates and values of
  the running median estimator (the red line in the plot).
  Its element \code{gg} is the plot object (see examples).
  Depending on the value of \code{plot}, the method can (and by default does) have a side effect,
  which is to print \code{gg} on the active graphics device.
}

\author{Wolfgang Huber}

\examples{
  data("kidney")
  log.na <- function(x) log(ifelse(x>0, x, NA))

  exprs(kidney) <- log.na(exprs(kidney))
  msd <- meanSdPlot(kidney)

  ## The `ggplot` object is returned in list element `gg`, here is an example of how to modify the plot
  library("ggplot2")
  msd$gg +
    ggtitle("Hello world") +
    scale_fill_gradient(low = "yellow", high = "darkred") +
    scale_y_continuous(limits = c(0, 7))

  ## Try this out with not log-transformed data, vsn2-transformed data, the lymphoma data, your data ...
}
\keyword{hplot}
\keyword{methods}

\name{scalingFactorTransformation}
\alias{scalingFactorTransformation}

\title{The transformation that is applied to the scaling parameter of
  the vsn model}
\description{The transformation that is applied to the scaling parameter of
  the vsn model}
\usage{
scalingFactorTransformation(b)
}

\arguments{
  \item{b}{Real vector.}
}
\value{
  A real vector of same length as b, with transformation \code{f}
  applied (see
  vignette \emph{Likelihood Calculations for vsn}).
}

\author{Wolfgang Huber}

\examples{
b  = seq(-3, 2, length=20)
fb = scalingFactorTransformation(b)
if(interactive())
  plot(b, fb, type="b", pch=16)
}

\name{kidney}
\alias{kidney}
\docType{data}
\title{Intensity data for one cDNA slide with two adjacent tissue samples
  from a nephrectomy (kidney)}
\description{Intensity data for one cDNA slide with two adjacent tissue samples
  from a nephrectomy (kidney)}
\usage{data(kidney)}

\format{\code{kidney} is an
  \code{\link[Biobase:class.ExpressionSet]{ExpressionSet}}
  containing the data from one cDNA
  chip. The 8704x2 matrix \code{exprs(kidney)} contains the
  spot intensities for the red (635 nm) and green color channels
  (532 nm) respectively. For each spot, a background estimate from a
  surrounding region was subtracted.
}

\details{The chip was produced in 2001 by Holger Sueltmann at
  the Division of Molecular Genome Analysis at the German Cancer
  Research Center in Heidelberg.}

\references{Huber W, Boer JM, von Heydebreck A, Gunawan B, Vingron M,
  Fuzesi L, Poustka A, Sueltmann H. Transcription profiling of renal
  cell carcinoma. Verh Dtsch Ges Pathol. 2002;86:153-64.
  PMID: 12647365}

\examples{
 data("kidney")
 plot(exprs(kidney), pch = ".", log = "xy")
 abline(a = 0, b = 1, col = "blue")  
}
\keyword{datasets}

\name{logLik-methods}
\alias{logLik-methods}
\alias{logLik,vsnInput-method}
\alias{plotVsnLogLik}

\title{Calculate the log likelihood and its gradient for
  the vsn model}
\description{\code{logLik} calculates the log likelihood and its gradient
  for the vsn model. \code{plotVsnLogLik} makes a false color plot for
  a 2D section of the likelihood landscape.}
\usage{
\S4method{logLik}{vsnInput}(object, p, mu = numeric(0), sigsq=as.numeric(NA), calib="affine")

plotVsnLogLik(object,
              p,
              whichp = 1:2,
              expand = 1,
              ngrid = 31L,
              fun = logLik,
              main = "log likelihood",
              ...)
}
\arguments{
  \item{object}{A \code{\link{vsnInput}} object.}
  \item{p}{For \code{plotVsnLogLik}, a vector or a 3D array with the
    point in parameter space around which to plot the likelihood.
    For \code{logLik}, a matrix whose columns are the set of parameters at which
    the likelihoods are to be evaluated.}
  \item{mu}{Numeric vector of length 0 or \code{nrow(object)}.
    If the length is 0, there is no reference and \code{sigsq} must be
    \code{NA} (the default value). See \code{\link{vsn2}}.}
  \item{sigsq}{Numeric scalar.}
  \item{calib}{as in \code{\link{vsn2}}.}
  \item{whichp}{Numeric vector of length 2, with the indices of those
    two parameters in \code{p} along which the section is to be taken.}
  \item{expand}{Numeric vector of length 1 or 2 with expansion factors
    for the plot range. The range is auto-calculated using a heuristic,
    but manual adjustment can be useful; see example.}
  \item{ngrid}{Integer scalar, the grid size.}
  \item{fun}{Function to use for log-likelihood calculation. This
    parameter is exposed only for testing purposes.}
  \item{main}{This parameter is passed on \code{levelplot}.}
  \item{...}{Arguments that get passed on to \code{fun}, use this for
    \code{mu}, \code{sigsq}, \code{calib}.}
}
\value{
For \code{logLik}, a numeric matrix of size \code{nrow(p)+1} by \code{ncol(p)}.
Its columns correspond to the columns of \code{p}.
Its first row are the likelihood values, its rows \code{2...nrow(p)+1}
contain the gradients.
If \code{mu} and \code{sigsq} are
specified, the ordinary negative log likelihood is calculated using these
parameters as given. If they are not specified, the profile negative log likelihood
is calculated.

For \code{plotVsnLogLik}, a dataframe with the 2D grid coordinates and
log likelihood values.
}

\details{
  \code{logLik} is an R interface to the likelihood computations in vsn (which are done in C).
}

\author{Wolfgang Huber}
\seealso{\code{\link{vsn2}}}

\examples{

data("kidney")

v = new("vsnInput", x=exprs(kidney),
  pstart=array(as.numeric(NA), dim=c(1, ncol(kidney), 2)))
 
fit = vsn2(kidney)
print(coef(fit))

p = sapply(seq(-1, 1, length=31), function(f) coef(fit)+c(0,0,f,0))

ll = logLik(v, p)

plot(p[3, ], ll[1, ], type="l", xlab=expression(b[1]), ylab=expression(-log(L)))
abline(v=coef(fit)[3], col="red")

plotVsnLogLik(v, coef(fit), whichp=c(1,3), expand=0.2)
}

\name{vsn2}
\alias{vsn2}
\alias{vsn2-methods}
\alias{vsn2,matrix-method}
\alias{vsn2,numeric-method}
\alias{vsn2,ExpressionSet-method}
\alias{vsn2,AffyBatch-method}
\alias{vsn2,RGList-method}
\alias{vsn2,NChannelSet-method}
\alias{coerce,RGList,NChannelSet-method}
\alias{vsnMatrix}

\title{Fit the vsn model}
\description{\code{vsn2} fits the vsn model to the data 
  in \code{x} and returns a \code{\linkS4class{vsn}} object with
  the fit parameters and the transformed data matrix.
  The data are, typically, feature intensity readings from a
  microarray, but this function may also be useful for other kinds of
  intensity data that obey an additive-multiplicative error model.
  To obtain an object of the same class as \code{x}, containing
  the normalised data and the same metdata as \code{x}, use
  \preformatted{
    fit = vsn2(x, ...)
    nx = predict(fit, newdata=x)
  }
  or the wrapper \code{\link{justvsn}}.
  Please see the vignette \emph{Introduction to vsn}.
}

\usage{
vsnMatrix(x,
          reference,
          strata,
          lts.quantile = 0.9,
          subsample    = 0L,
          verbose      = interactive(),
          returnData   = TRUE,
          calib        = "affine",
          pstart,
          minDataPointsPerStratum = 42L,
          optimpar     = list(),
          defaultpar   = list(factr=5e7, pgtol=2e-4, maxit=60000L,
                              trace=0L, cvg.niter=7L, cvg.eps=0))

\S4method{vsn2}{ExpressionSet}(x, reference, strata, ...)

\S4method{vsn2}{AffyBatch}(x, reference, strata, subsample, ...)

\S4method{vsn2}{NChannelSet}(x, reference, strata, backgroundsubtract=FALSE,
       foreground=c("R","G"), background=c("Rb", "Gb"), ...)

\S4method{vsn2}{RGList}(x, reference, strata, ...)
}
\arguments{
  \item{x}{An object containing the data to which the model is fitted.}
  \item{reference}{Optional, a \code{\linkS4class{vsn}} object from
    a previous fit. If this argument is specified, the data in \code{x}
    are normalized "towards" an existing set of reference arrays whose
    parameters are stored in the object \code{reference}. If this
    argument is not specified, then the data in \code{x} are normalized
    "among themselves". See Details for a more precise explanation.}
  \item{strata}{Optional, a \code{factor} or \code{integer}
    whose length is \code{nrow(x)}. It can
    be used for stratified normalization (i.e. separate offsets \eqn{a} and
    factors \eqn{b} for each level of \code{strata}). If missing, all
    rows of \code{x} are assumed to come from one stratum.
    If \code{strata} is an integer, its values must cover the range
    \eqn{1,\ldots,n}, where \eqn{n} is the number of strata.}
  \item{lts.quantile}{Numeric of length 1. The quantile that is used for
    the resistant least trimmed sum of squares regression. Allowed
    values are between 0.5 and 1. A value of 1 corresponds to ordinary
    least sum of squares regression.}
  \item{subsample}{Integer of length 1. If its value is greater than 0,
    the model parameters are
    estimated from a subsample of the data of size \code{subsample}
    only, yet the fitted transformation is
    then applied to all data. For large datasets, this can substantially
    reduce the CPU time and memory consumption at a negligible loss of
    precision. Note that the \code{AffyBatch} method of \code{vsn2} sets a value of
    \code{30000} for this parameter if it is missing from the function
    call - which is different from the behaviour of the other methods.}
  \item{backgroundsubtract}{Logical of length 1: should
    local background estimates be subtracted before fitting vsn?}
  \item{foreground, background}{Aligned character vectors of the same length,
    naming the channels of \code{x} that should be used
    as foreground and background values.}
  \item{verbose}{Logical. If TRUE, some messages are printed.}
  \item{returnData}{Logical. If TRUE, the transformed data are returned
    in a slot of the resulting \code{\linkS4class{vsn}} object.
    Setting this option to \code{FALSE} allows saving memory
    if the data are not needed.}
  \item{calib}{Character of length 1. Allowed values are \code{affine}
    and \code{none}. The default, \code{affine}, corresponds to the
    behaviour in package versions <= 3.9, and to what is described in
    references [1] and [2]. The option \code{none} is an experimental
    new feature, in which no affine calibration is performed and only
    two global variance stabilisation transformation parameters \code{a}
    and \code{b} are fitted. This functionality might be useful in
    conjunction with other calibration methods, such as quantile
    normalisation - see the vignette \emph{Introduction to vsn}.}
  \item{pstart}{Optional, a three-dimensional numeric array that
    specifies start values for the iterative parameter
    estimation algorithm.
    If not specified, the function tries to guess useful start values.
    The first dimension corresponds to the levels of \code{strata},
    the second dimension to the columns of \code{x} and the third dimension
    must be 2, corresponding to offsets and factors.}
  \item{minDataPointsPerStratum}{The minimum number of data points per
    stratum. Normally there is no need for the user to change this;
    refer to the vignette for further documentation.}
  \item{optimpar}{Optional, a list with parameters for the likelihood
    optimisation algorithm. Default parameters are taken from
    \code{defaultpar}. See details.}
  \item{defaultpar}{The default parameters  for the likelihood
    optimisation algorithm. Values in \code{optimpar} take precedence
    over those in \code{defaultpar}. The purpose of this argument is to
    expose the default values in this manual page - it is not
    intended to be changed, please use \code{optimpar} for that.}
  \item{...}{Arguments that get passed on to \code{vsnMatrix}.}
}

\value{An object of class \code{\linkS4class{vsn}}.}

\section{Note on overall scale and location of the glog
  transformation}{
  The data are returned on a \eqn{glog} scale to base 2. More precisely,
  the transformed data are subject to the transformation
  \eqn{glog_2(f(b)*x+a) + c}, where the function
  \eqn{glog_2(u) = log_2(u+\sqrt{u*u+1}) = asinh(u)/\log(2)} is called the
  generalised logarithm, the offset \eqn{a} and the scaling parameter
  \eqn{b} are the fitted model parameters
  (see references), and \eqn{f(x)=\exp(x)} is a parameter transformation that
  allows ensuring positivity of the factor in front of \eqn{x} while
  using an unconstrained optimisation over \eqn{b} [4].
  The overall offset \eqn{c} is computed from the \eqn{b}'s such that for
  large \eqn{x} the transformation approximately corresponds to the
  \eqn{\log_2} function. This is done separately for each stratum, but with the
  same value across arrays. More precisely, if the element \code{b[s,i]}
  of the array \emph{b} is the scaling parameter for the \code{s}-th
  stratum and the \code{i}-th array, then \code{c[s]} is computed as
  \code{log2(2*f(mean(b[,i])))}.
  The offset \emph{c} is inconsequential for all differential
  expression calculations, but many users like to see the data in a
  range that they are familiar with.
}

\section{Specific behaviour of the different methods}{
  \code{vsn2} methods exist for
  \code{\link[Biobase:class.ExpressionSet]{ExpressionSet}},
  \code{\link[Biobase:class.NChannelSet]{NChannelSet}},
  \code{\link[affy:AffyBatch-class]{AffyBatch}} (from the \code{affy} package),
  \code{\link[limma:rglist]{RGList}} (from the \code{limma} package),
  \code{matrix} and \code{numeric}.
  If \code{x} is an \code{\link[Biobase:class.NChannelSet]{NChannelSet}}, then
  \code{vsn2} is applied to the matrix that is obtained
  by horizontally concatenating the color channels.
  Optionally, available background estimates can be subtracted before.
  If \code{x} is an \code{\link[limma:rglist]{RGList}}, it is
  converted into an \code{NChannelSet}
  using a copy of Martin Morgan's code for \code{RGList} to
  \code{NChannelSet} coercion, then the \code{NChannelSet} method is called.
}

\section{Standalone versus reference normalisation}{
  If the \code{reference} argument is \emph{not} specified, then the model
  parameters \eqn{\mu_k} and \eqn{\sigma} are fit from the data in \code{x}.
  This is the mode of operation described in [1]
  and that was the only option in versions 1.X of this package.
  If \code{reference} is specified, the model parameters
  \eqn{\mu_k} and \eqn{\sigma} are taken from it.
  This allows for 'incremental' normalization [4].}

\section{Convergence of the iterative likelihood optimisation}{
  \code{L-BFGS-B} uses three termination criteria:
  \enumerate{
    \item \code{(f_k - f_{k+1}) / max(|f_k|, |f_{k+1}|, 1) <= factr * epsmch}
      where \code{epsmch} is the machine precision.
    \item \code{|gradient| < pgtol}
    \item \code{iterations > maxit}
  }
  These are set by the elements \code{factr}, \code{pgtol} and
  \code{maxit} of \code{optimpar}. The remaining elements are
  \describe{
    \item{\code{trace}}{An integer between 0 and 6, indicating the
      verbosity level of \code{L-BFGS-B}, higher values
      create more output.}
    \item{\code{cvg.niter}}{The number of iterations to be used in the least
      trimmed sum of squares regression.}
    \item{\code{cvg.eps}}{Numeric. A convergence threshold for the least
      trimmed sum of squares regression.}
  }
}

\seealso{\code{\link{justvsn}}, \code{\link[vsn:vsn2trsf]{predict}}}

\references{
[1] Variance stabilization applied to microarray data
calibration and to the quantification of differential expression,
Wolfgang Huber, Anja von Heydebreck, Holger Sueltmann, Annemarie
Poustka, Martin Vingron; Bioinformatics (2002) 18 Suppl.1 S96-S104.

[2] Parameter estimation for the calibration and variance stabilization 
of microarray data, 
Wolfgang Huber, Anja von Heydebreck, Holger Sueltmann, 
Annemarie Poustka, and Martin Vingron;  
Statistical Applications in Genetics and Molecular Biology (2003)
Vol. 2 No. 1, Article 3.
http://www.bepress.com/sagmb/vol2/iss1/art3.

[3] L-BFGS-B: Fortran Subroutines for Large-Scale Bound Constrained
Optimization, C. Zhu, R.H. Byrd, P. Lu and J. Nocedal,
Technical Report, Northwestern University (1996).

[4] Package vignette: Likelihood Calculations for vsn
}

\author{Wolfgang Huber}

\examples{
data("kidney")

fit = vsn2(kidney)                   ## fit
nkid = predict(fit, newdata=kidney)  ## apply fit

plot(exprs(nkid), pch=".")
abline(a=0, b=1, col="red")
}

\name{vsn2trsf}
\alias{predict,vsn-method}

\title{Apply the vsn transformation to data}
\description{Apply the vsn transformation to data.}
\usage{
\S4method{predict}{vsn}(object, newdata, strata=object@strata, log2scale=TRUE, useDataInFit=FALSE)
}

\arguments{
  \item{object}{An object of class \code{\linkS4class{vsn}} that
    contains transformation parameters and strata information, typically
    this is the result of a previous call to \code{vsn2}.}
  \item{newdata}{Object of class
    \code{\link[Biobase:class.ExpressionSet]{ExpressionSet}},
    \code{\link[Biobase:class.NChannelSet]{NChannelSet}},
    \code{\link[affy:AffyBatch-class]{AffyBatch}} (from the \code{affy} package),
    \code{\link[limma:rglist]{RGList}} (from the \code{limma} package),
    \code{matrix} or \code{numeric}, with the data to which the
    fit is to be applied to.}
  \item{strata}{Optional, a \code{factor} or \code{integer}
    that aligns with the rows of \code{newdata}; see
    the \code{strata} argument of \code{\link{vsn2}}.}
  \item{log2scale}{If \code{TRUE}, the data are returned on the glog
    scale to base 2, and an overall offset c is added (see \emph{Value}
    section of the \code{\link{vsn2}} manual page). If \code{FALSE}, the
    data are returned on the glog scale to base e, and no offset is
    added.}
  \item{useDataInFit}{If \code{TRUE}, then no transformation is
    attempted and the data stored in \code{object} is transferred
    appropriately into resulting object, which otherwise preserves the
    class and metadata of \code{newdata}. This option exists to increase
    performance in constructs like
\preformatted{
       fit = vsn2(x, ...)
       nx = predict(fit, newdata=x)
  }
    and is used, for example, in the \code{\link{justvsn}} function.
  }
}

\value{
  An object typically of the same class as \code{newdata}. There are two
  exceptions: if \code{newdata} is an
  \code{\link[limma:rglist]{RGList}}, the return value is an
  \code{\link[Biobase:class.NChannelSet]{NChannelSet}}, and
  if \code{newdata} is numeric, the return value is a \code{matrix} with 1
  column.
}

\author{Wolfgang Huber}

\examples{
data("kidney")

## nb: for random subsampling, the 'subsample' argument of vsn
##   provides an easier way to do this
fit = vsn2(kidney[sample(nrow(kidney), 500), ])
tn = predict(fit, newdata=exprs(kidney))

}

\name{vsn}
\docType{class}
\alias{class:vsn}
\alias{vsn-class}

\alias{[,vsn-method} 
\alias{dim,vsn-method} 
\alias{nrow,vsn-method} 
\alias{ncol,vsn-method} 
\alias{show,vsn-method} 
\alias{exprs,vsn-method} 
\alias{coef,vsn-method} 
\alias{coefficients,vsn-method} 

\title{
  Class to contain result of a vsn fit
}

\description{
  Class to contain result of a vsn fit
}

\section{Creating Objects}{
  \code{new("vsn")}
  \code{vsn2(x)} with \code{x} being an
  \code{\link[Biobase:class.ExpressionSet]{ExpressionSet}}. 
}

\section{Slots}{
 \describe{
  \item{\code{coefficients}:}{A 3D array of size (number of strata) x (number of
    columns of the data matrix) x 2. It contains the fitted
    normalization parameters (see vignette).}
  \item{\code{strata}:}{A factor of length 0 or n. If its length is
    n, then its levels correspond to different normalization strata
    (see vignette).}
  \item{\code{mu}:}{A numeric vector of length n with the fitted
    parameters \eqn{\hat{\mu}_k}, for \eqn{k=1,...,n}.}
  \item{\code{sigsq}:}{A numeric scalar, \eqn{\hat{\sigma}^2}.}
  \item{\code{hx}:}{A numeric matrix with 0 or n rows. If the number of
    rows is n, then \code{hx} contains the transformed data matrix.}
  \item{\code{lbfgsb}:}{An integer scalar containing the return code
    from the L-BFGS-B optimizer.}
  \item{\code{hoffset}:}{Numeric scalar, the overall offset \eqn{c}- see
 manual page of \code{\link{vsn2}}.} 
  \item{\code{calib}:}{Character of length 1, see manual page of
 \code{\link{vsn2}}.} 
}}


\section{Methods}{
  \describe{
    \item{\code{[}}{Subset}
    \item{\code{dim}}{Get dimensions of data matrix.}
    \item{\code{nrow}}{Get number of rows of data matrix.}
    \item{\code{ncol}}{Get number of columns of data matrix.}
    \item{\code{show}}{Print a summary of the object}
    \item{\code{exprs}}{Accessor to slot \code{hx}.}
    \item{\code{coef}, \code{coefficients}}{Accessors to slot
  \code{coefficients}.} 
}}


\author{Wolfgang Huber}

\seealso{
  \code{\link{vsn2}}
}

\examples{
  data("kidney")
  v = vsn2(kidney)
  show(v)
  dim(v)
  v[1:10, ]
}

\keyword{classes}

\name{lymphoma}
\alias{lymphoma}
\docType{data}
\title{Intensity data for 8 cDNA slides with CLL and DLBL samples from
  the Alizadeh et al. paper in Nature 2000}
\description{8 cDNA chips from Alizadeh lymphoma paper}
\usage{data(lymphoma)}

\format{\code{lymphoma} is an \code{\link[Biobase:class.ExpressionSet]{ExpressionSet}}
  containing the data from 8 chips
  from the lymphoma data set by Alizadeh et al. (see references). Each
  chip represents two samples: on color channel 1 (CH1, Cy3, green) the
  common reference sample, and on color channel 2 (CH2, Cy5, red) the
  various disease samples. See \code{pData(lymphoma)}. The 9216x16
  matrix \code{exprs(lymphoma)} contains the background-subtracted spot
  intensities (CH1I-CH1B and CH2I-CH2B, respectively). 
}

\details{
The chip intensity files were downloaded from the Stanford
microarray database. Starting from the link below, this was done by
following the links \emph{Published Data} -> 
\emph{Alizadeh AA, et al. (2000) Nature 403(6769):503-11} -> 
\emph{Data in SMD} -> \emph{Display Data}, and selecting the following 
8 slides:
\tabular{l}{
lc7b019\cr
lc7b047\cr
lc7b048\cr
lc7b056\cr
lc7b057\cr
lc7b058\cr
lc7b069\cr
lc7b070
}
Then, the script \code{makedata.R} from the \code{scripts} subdirectory
of this package was run to generate the \R data object.
}

\references{A. Alizadeh et al., Distinct types of diffuse large B-cell 
lymphoma identified by gene expression profiling. Nature 403(6769):503-11, 
Feb 3, 2000.}

\source{http://genome-www5.stanford.edu/MicroArray/SMD}

\examples{
   data("lymphoma")
   lymphoma
   pData(lymphoma)
}

\keyword{datasets}

\name{normalize.AffyBatch.vsn}
\alias{normalize.AffyBatch.vsn}
\title{Wrapper for vsn to be used as a normalization method with expresso}
\description{Wrapper for \code{\link{vsn2}} to be used as a normalization
  method with the expresso function of the package affy. The expresso
  function is deprecated, consider using \code{\link{justvsn}}
  instead. The normalize.AffyBatch.vsn can still be useful on its own,
  as it provides some additional control of the normalization process
  (fitting on subsets, alternate transform parameters). }
\usage{
  normalize.AffyBatch.vsn(
     abatch,
     reference,
     strata = NULL,
     subsample = if (nrow(exprs(abatch))>30000L) 30000L else 0L,
     subset,
     log2scale = TRUE,
     log2asymp=FALSE,
     ...)}
\arguments{
  \item{abatch}{An object of type
    \code{\link[affy:AffyBatch-class]{AffyBatch}}.}
  \item{reference}{Optional, a 'vsn' object from a previous fit. If this
    argument is specified, the data in 'x' are normalized "towards" an
    existing set of reference arrays whose parameters are stored in the
    object 'reference'. If this argument is not specified, then the data
    in 'x' are normalized "among themselves". See  \code{\link{vsn2}}
    for details.}
  \item{strata}{The 'strata' functionality is not supported, the
    parameter is ignored.}
  \item{subsample}{Is passed on to \code{\link{vsn2}}.}
  \item{subset}{This allows the specification of a subset of expression
    measurements to be used for the vsn fit. The transformation with the
    parameters of this fit is then, however, applied to the whole
    dataset. This is useful for excluding expression measurements that
    are known to be differentially expressed or control probes that may
    not match the vsn model, thus avoiding that they influence the
    normalization process. This operates at the level of probesets, not
    probes. Both 'subset' and 'subsample' can be used together.}
  \item{log2scale}{If TRUE, this will perform a global affine transform on
    the data to put them on a similar scale as the original
    non-transformed data. Many users prefer this.  Fold-change estimates
    are not affected by this transform. In some situations,
    however, it may be helpful to turn this off, e.g., when comparing
    independently normalized subsets of the data.}
  \item{log2asymp}{If TRUE, this will perform a global affine transform on
    the data to make the generalized log (asinh) transform be
    asymptotically identical to a log base 2 transform. Some people find
    this helpful. Only \bold{one} of 'log2scale' or 'log2asymp' can be
    set to TRUE. Fold-change estimates are not affected by this
    transform.}
  \item{...}{Further parameters for \code{\link{vsn2}}.}
}
 
\details{Please refer to the \emph{Details} and \emph{References}
  sections of the
  man page for \code{\link{vsn2}} for more details about this method.

  \bold{Important note}: after calling \code{\link{vsn2}}, the function
  \code{normalize.AffyBatch.vsn} \bold{exponentiates} the data (base 2).
  This is done in order to make the behavior of this function similar to the
  other normalization methods in affy. That packages uses the convention
  of taking the logarithm to base in
  subsequent analysis steps (e.g. in \code{\link[stats]{medpolish}}).
}

\value{An object of class \code{\link[affy:AffyBatch-class]{AffyBatch}}.
  The \code{vsn} object returned, which can be used as \code{reference} for
  subsequent fits, is provided by
  \code{description(abatch)@preprocessing$vsnReference}.
}
\author{D. P. Kreil \url{http://bioinf.boku.ac.at/},
        Wolfgang Huber}

\seealso{\code{\link{vsn2}}}
\examples{
## Please see vignette.
}
